@@ -59,8 +59,6 @@
  * to avoid polling.
  *
  * Last reviewed on 2008-12-17 (0.10.22)
- *
- * Since: 0.10.22
  */
 
 #ifdef HAVE_CONFIG_H
@@ -76,8 +74,6 @@
 #include "gstapp-marshal.h"
 #include "gstappsink.h"
 
-#include "gst/glib-compat-private.h"
-
 struct _GstAppSinkPrivate
 {
   GstCaps *caps;
@@ -86,8 +82,8 @@ struct _GstAppSinkPrivate
   guint max_buffers;
   gboolean drop;
 
-  GCond *cond;
-  GMutex *mutex;
+  GCond cond;
+  GMutex mutex;
   GQueue *queue;
   GstBuffer *preroll;
   GstCaps *preroll_caps;
@@ -245,7 +241,8 @@ gst_app_sink_class_init (GstAppSinkClass * klass)
   gst_app_sink_signals[SIGNAL_NEW_PREROLL] =
       g_signal_new ("new-preroll", G_TYPE_FROM_CLASS (klass), G_SIGNAL_RUN_LAST,
       G_STRUCT_OFFSET (GstAppSinkClass, new_preroll),
-      NULL, NULL, g_cclosure_marshal_VOID__VOID, G_TYPE_NONE, 0, G_TYPE_NONE);
+      NULL, NULL, __gst_app_marshal_ENUM__VOID, GST_TYPE_FLOW_RETURN, 0,
+      G_TYPE_NONE);
   /**
    * GstAppSink::new-sample:
    * @appsink: the appsink element that emited the signal
@@ -265,7 +262,8 @@ gst_app_sink_class_init (GstAppSinkClass * klass)
   gst_app_sink_signals[SIGNAL_NEW_SAMPLE] =
       g_signal_new ("new-sample", G_TYPE_FROM_CLASS (klass), G_SIGNAL_RUN_LAST,
       G_STRUCT_OFFSET (GstAppSinkClass, new_sample),
-      NULL, NULL, g_cclosure_marshal_VOID__VOID, G_TYPE_NONE, 0, G_TYPE_NONE);
+      NULL, NULL, __gst_app_marshal_ENUM__VOID, GST_TYPE_FLOW_RETURN, 0,
+      G_TYPE_NONE);
 
   /**
    * GstAppSink::pull-preroll:
@@ -322,7 +320,7 @@ gst_app_sink_class_init (GstAppSinkClass * klass)
           pull_sample), NULL, NULL, __gst_app_marshal_BOXED__VOID,
       GST_TYPE_SAMPLE, 0, G_TYPE_NONE);
 
-  gst_element_class_set_details_simple (element_class, "AppSink",
+  gst_element_class_set_metadata (element_class, "AppSink",
       "Generic/Sink", "Allow the application to get access to raw buffer",
       "David Schleef <ds@schleef.org>, Wim Taymans <wim.taymans@gmail.com>");
 
@@ -355,8 +353,8 @@ gst_app_sink_init (GstAppSink * appsink)
       G_TYPE_INSTANCE_GET_PRIVATE (appsink, GST_TYPE_APP_SINK,
       GstAppSinkPrivate);
 
-  priv->mutex = g_mutex_new ();
-  priv->cond = g_cond_new ();
+  g_mutex_init (&priv->mutex);
+  g_cond_init (&priv->cond);
   priv->queue = g_queue_new ();
 
   priv->emit_signals = DEFAULT_PROP_EMIT_SIGNALS;
@@ -384,13 +382,13 @@ gst_app_sink_dispose (GObject * obj)
 
   GST_OBJECT_UNLOCK (appsink);
 
-  g_mutex_lock (priv->mutex);
+  g_mutex_lock (&priv->mutex);
   while ((queue_obj = g_queue_pop_head (priv->queue)))
     gst_mini_object_unref (queue_obj);
   gst_buffer_replace (&priv->preroll, NULL);
   gst_caps_replace (&priv->preroll_caps, NULL);
   gst_caps_replace (&priv->last_caps, NULL);
-  g_mutex_unlock (priv->mutex);
+  g_mutex_unlock (&priv->mutex);
 
   G_OBJECT_CLASS (parent_class)->dispose (obj);
 }
@@ -401,8 +399,8 @@ gst_app_sink_finalize (GObject * obj)
   GstAppSink *appsink = GST_APP_SINK_CAST (obj);
   GstAppSinkPrivate *priv = appsink->priv;
 
-  g_mutex_free (priv->mutex);
-  g_cond_free (priv->cond);
+  g_mutex_clear (&priv->mutex);
+  g_cond_clear (&priv->cond);
   g_queue_free (priv->queue);
 
   G_OBJECT_CLASS (parent_class)->finalize (obj);
@@ -474,11 +472,11 @@ gst_app_sink_unlock_start (GstBaseSink * bsink)
   GstAppSink *appsink = GST_APP_SINK_CAST (bsink);
   GstAppSinkPrivate *priv = appsink->priv;
 
-  g_mutex_lock (priv->mutex);
+  g_mutex_lock (&priv->mutex);
   GST_DEBUG_OBJECT (appsink, "unlock start");
   priv->unlock = TRUE;
-  g_cond_signal (priv->cond);
-  g_mutex_unlock (priv->mutex);
+  g_cond_signal (&priv->cond);
+  g_mutex_unlock (&priv->mutex);
 
   return TRUE;
 }
@@ -489,11 +487,11 @@ gst_app_sink_unlock_stop (GstBaseSink * bsink)
   GstAppSink *appsink = GST_APP_SINK_CAST (bsink);
   GstAppSinkPrivate *priv = appsink->priv;
 
-  g_mutex_lock (priv->mutex);
+  g_mutex_lock (&priv->mutex);
   GST_DEBUG_OBJECT (appsink, "unlock stop");
   priv->unlock = FALSE;
-  g_cond_signal (priv->cond);
-  g_mutex_unlock (priv->mutex);
+  g_cond_signal (&priv->cond);
+  g_mutex_unlock (&priv->mutex);
 
   return TRUE;
 }
@@ -510,7 +508,7 @@ gst_app_sink_flush_unlocked (GstAppSink * appsink)
   while ((obj = g_queue_pop_head (priv->queue)))
     gst_mini_object_unref (obj);
   priv->num_buffers = 0;
-  g_cond_signal (priv->cond);
+  g_cond_signal (&priv->cond);
 }
 
 static gboolean
@@ -519,12 +517,12 @@ gst_app_sink_start (GstBaseSink * psink)
   GstAppSink *appsink = GST_APP_SINK_CAST (psink);
   GstAppSinkPrivate *priv = appsink->priv;
 
-  g_mutex_lock (priv->mutex);
+  g_mutex_lock (&priv->mutex);
   GST_DEBUG_OBJECT (appsink, "starting");
   priv->flushing = FALSE;
   priv->started = TRUE;
   gst_segment_init (&priv->last_segment, GST_FORMAT_TIME);
-  g_mutex_unlock (priv->mutex);
+  g_mutex_unlock (&priv->mutex);
 
   return TRUE;
 }
@@ -535,14 +533,14 @@ gst_app_sink_stop (GstBaseSink * psink)
   GstAppSink *appsink = GST_APP_SINK_CAST (psink);
   GstAppSinkPrivate *priv = appsink->priv;
 
-  g_mutex_lock (priv->mutex);
+  g_mutex_lock (&priv->mutex);
   GST_DEBUG_OBJECT (appsink, "stopping");
   priv->flushing = TRUE;
   priv->started = FALSE;
   gst_app_sink_flush_unlocked (appsink);
   gst_caps_replace (&priv->preroll_caps, NULL);
   gst_caps_replace (&priv->last_caps, NULL);
-  g_mutex_unlock (priv->mutex);
+  g_mutex_unlock (&priv->mutex);
 
   return TRUE;
 }
@@ -553,11 +551,11 @@ gst_app_sink_setcaps (GstBaseSink * sink, GstCaps * caps)
   GstAppSink *appsink = GST_APP_SINK_CAST (sink);
   GstAppSinkPrivate *priv = appsink->priv;
 
-  g_mutex_lock (priv->mutex);
+  g_mutex_lock (&priv->mutex);
   GST_DEBUG_OBJECT (appsink, "receiving CAPS");
   g_queue_push_tail (priv->queue, gst_event_new_caps (caps));
   gst_caps_replace (&priv->preroll_caps, caps);
-  g_mutex_unlock (priv->mutex);
+  g_mutex_unlock (&priv->mutex);
 
   return TRUE;
 }
@@ -570,17 +568,17 @@ gst_app_sink_event (GstBaseSink * sink, GstEvent * event)
 
   switch (event->type) {
     case GST_EVENT_SEGMENT:
-      g_mutex_lock (priv->mutex);
+      g_mutex_lock (&priv->mutex);
       GST_DEBUG_OBJECT (appsink, "receiving SEGMENT");
       g_queue_push_tail (priv->queue, gst_event_ref (event));
-      g_mutex_unlock (priv->mutex);
+      g_mutex_unlock (&priv->mutex);
       break;
     case GST_EVENT_EOS:
-      g_mutex_lock (priv->mutex);
+      g_mutex_lock (&priv->mutex);
       GST_DEBUG_OBJECT (appsink, "receiving EOS");
       priv->is_eos = TRUE;
-      g_cond_signal (priv->cond);
-      g_mutex_unlock (priv->mutex);
+      g_cond_signal (&priv->cond);
+      g_mutex_unlock (&priv->mutex);
 
       /* emit EOS now */
       if (priv->callbacks.eos)
@@ -595,10 +593,10 @@ gst_app_sink_event (GstBaseSink * sink, GstEvent * event)
       GST_DEBUG_OBJECT (appsink, "received FLUSH_START");
       break;
     case GST_EVENT_FLUSH_STOP:
-      g_mutex_lock (priv->mutex);
+      g_mutex_lock (&priv->mutex);
       GST_DEBUG_OBJECT (appsink, "received FLUSH_STOP");
       gst_app_sink_flush_unlocked (appsink);
-      g_mutex_unlock (priv->mutex);
+      g_mutex_unlock (&priv->mutex);
       break;
     default:
       break;
@@ -609,33 +607,37 @@ gst_app_sink_event (GstBaseSink * sink, GstEvent * event)
 static GstFlowReturn
 gst_app_sink_preroll (GstBaseSink * psink, GstBuffer * buffer)
 {
-  GstFlowReturn res = GST_FLOW_OK;
+  GstFlowReturn res;
   GstAppSink *appsink = GST_APP_SINK_CAST (psink);
   GstAppSinkPrivate *priv = appsink->priv;
   gboolean emit;
 
-  g_mutex_lock (priv->mutex);
+  g_mutex_lock (&priv->mutex);
   if (priv->flushing)
     goto flushing;
 
   GST_DEBUG_OBJECT (appsink, "setting preroll buffer %p", buffer);
   gst_buffer_replace (&priv->preroll, buffer);
 
-  g_cond_signal (priv->cond);
+  g_cond_signal (&priv->cond);
   emit = priv->emit_signals;
-  g_mutex_unlock (priv->mutex);
+  g_mutex_unlock (&priv->mutex);
 
-  if (priv->callbacks.new_preroll)
+  if (priv->callbacks.new_preroll) {
     res = priv->callbacks.new_preroll (appsink, priv->user_data);
-  else if (emit)
-    g_signal_emit (appsink, gst_app_sink_signals[SIGNAL_NEW_PREROLL], 0);
+  } else {
+    res = GST_FLOW_OK;
+    if (emit)
+      g_signal_emit (appsink, gst_app_sink_signals[SIGNAL_NEW_PREROLL], 0,
+          &res);
+  }
 
   return res;
 
 flushing:
   {
     GST_DEBUG_OBJECT (appsink, "we are flushing");
-    g_mutex_unlock (priv->mutex);
+    g_mutex_unlock (&priv->mutex);
     return GST_FLOW_FLUSHING;
   }
 }
@@ -693,7 +695,7 @@ gst_app_sink_render (GstBaseSink * psink, GstBuffer * buffer)
   gboolean emit;
 
 restart:
-  g_mutex_lock (priv->mutex);
+  g_mutex_lock (&priv->mutex);
   if (priv->flushing)
     goto flushing;
 
@@ -724,7 +726,7 @@ restart:
 
       if (priv->unlock) {
         /* we are asked to unlock, call the wait_preroll method */
-        g_mutex_unlock (priv->mutex);
+        g_mutex_unlock (&priv->mutex);
         if ((ret = gst_base_sink_wait_preroll (psink)) != GST_FLOW_OK)
           goto stopping;
 
@@ -733,7 +735,7 @@ restart:
       }
 
       /* wait for a buffer to be removed or flush */
-      g_cond_wait (priv->cond, priv->mutex);
+      g_cond_wait (&priv->cond, &priv->mutex);
       if (priv->flushing)
         goto flushing;
     }
@@ -741,21 +743,23 @@ restart:
   /* we need to ref the buffer when pushing it in the queue */
   g_queue_push_tail (priv->queue, gst_buffer_ref (buffer));
   priv->num_buffers++;
-  g_cond_signal (priv->cond);
+  g_cond_signal (&priv->cond);
   emit = priv->emit_signals;
-  g_mutex_unlock (priv->mutex);
+  g_mutex_unlock (&priv->mutex);
 
-  if (priv->callbacks.new_sample)
-    priv->callbacks.new_sample (appsink, priv->user_data);
-  else if (emit)
-    g_signal_emit (appsink, gst_app_sink_signals[SIGNAL_NEW_SAMPLE], 0);
-
-  return GST_FLOW_OK;
+  if (priv->callbacks.new_sample) {
+    ret = priv->callbacks.new_sample (appsink, priv->user_data);
+  } else {
+    ret = GST_FLOW_OK;
+    if (emit)
+      g_signal_emit (appsink, gst_app_sink_signals[SIGNAL_NEW_SAMPLE], 0, &ret);
+  }
+  return ret;
 
 flushing:
   {
     GST_DEBUG_OBJECT (appsink, "we are flushing");
-    g_mutex_unlock (priv->mutex);
+    g_mutex_unlock (&priv->mutex);
     return GST_FLOW_FLUSHING;
   }
 stopping:
@@ -820,8 +824,6 @@ gst_app_sink_query (GstBaseSink * bsink, GstQuery * query)
  * a copy of the caps structure. After calling this method, the sink will only
  * accept caps that match @caps. If @caps is non-fixed, you must check the caps
  * on the buffers to get the actual used caps.
- *
- * Since: 0.10.22
  */
 void
 gst_app_sink_set_caps (GstAppSink * appsink, const GstCaps * caps)
@@ -853,8 +855,6 @@ gst_app_sink_set_caps (GstAppSink * appsink, const GstCaps * caps)
  * Get the configured caps on @appsink.
  *
  * Returns: the #GstCaps accepted by the sink. gst_caps_unref() after usage.
- *
- * Since: 0.10.22
  */
 GstCaps *
 gst_app_sink_get_caps (GstAppSink * appsink)
@@ -886,8 +886,6 @@ gst_app_sink_get_caps (GstAppSink * appsink)
  * PLAYING state.
  *
  * Returns: %TRUE if no more samples can be pulled and the appsink is EOS.
- *
- * Since: 0.10.22
  */
 gboolean
 gst_app_sink_is_eos (GstAppSink * appsink)
@@ -899,7 +897,7 @@ gst_app_sink_is_eos (GstAppSink * appsink)
 
   priv = appsink->priv;
 
-  g_mutex_lock (priv->mutex);
+  g_mutex_lock (&priv->mutex);
   if (!priv->started)
     goto not_started;
 
@@ -910,14 +908,14 @@ gst_app_sink_is_eos (GstAppSink * appsink)
     GST_DEBUG_OBJECT (appsink, "we are not yet EOS");
     ret = FALSE;
   }
-  g_mutex_unlock (priv->mutex);
+  g_mutex_unlock (&priv->mutex);
 
   return ret;
 
 not_started:
   {
     GST_DEBUG_OBJECT (appsink, "we are stopped, return TRUE");
-    g_mutex_unlock (priv->mutex);
+    g_mutex_unlock (&priv->mutex);
     return TRUE;
   }
 }
@@ -930,8 +928,6 @@ not_started:
  * Make appsink emit the "new-preroll" and "new-sample" signals. This option is
  * by default disabled because signal emission is expensive and unneeded when
  * the application prefers to operate in pull mode.
- *
- * Since: 0.10.22
  */
 void
 gst_app_sink_set_emit_signals (GstAppSink * appsink, gboolean emit)
@@ -942,9 +938,9 @@ gst_app_sink_set_emit_signals (GstAppSink * appsink, gboolean emit)
 
   priv = appsink->priv;
 
-  g_mutex_lock (priv->mutex);
+  g_mutex_lock (&priv->mutex);
   priv->emit_signals = emit;
-  g_mutex_unlock (priv->mutex);
+  g_mutex_unlock (&priv->mutex);
 }
 
 /**
@@ -955,8 +951,6 @@ gst_app_sink_set_emit_signals (GstAppSink * appsink, gboolean emit)
  *
  * Returns: %TRUE if @appsink is emiting the "new-preroll" and "new-sample"
  * signals.
- *
- * Since: 0.10.22
  */
 gboolean
 gst_app_sink_get_emit_signals (GstAppSink * appsink)
@@ -968,9 +962,9 @@ gst_app_sink_get_emit_signals (GstAppSink * appsink)
 
   priv = appsink->priv;
 
-  g_mutex_lock (priv->mutex);
+  g_mutex_lock (&priv->mutex);
   result = priv->emit_signals;
-  g_mutex_unlock (priv->mutex);
+  g_mutex_unlock (&priv->mutex);
 
   return result;
 }
@@ -983,8 +977,6 @@ gst_app_sink_get_emit_signals (GstAppSink * appsink)
  * Set the maximum amount of buffers that can be queued in @appsink. After this
  * amount of buffers are queued in appsink, any more buffers will block upstream
  * elements until a sample is pulled from @appsink.
- *
- * Since: 0.10.22
  */
 void
 gst_app_sink_set_max_buffers (GstAppSink * appsink, guint max)
@@ -995,13 +987,13 @@ gst_app_sink_set_max_buffers (GstAppSink * appsink, guint max)
 
   priv = appsink->priv;
 
-  g_mutex_lock (priv->mutex);
+  g_mutex_lock (&priv->mutex);
   if (max != priv->max_buffers) {
     priv->max_buffers = max;
     /* signal the change */
-    g_cond_signal (priv->cond);
+    g_cond_signal (&priv->cond);
   }
-  g_mutex_unlock (priv->mutex);
+  g_mutex_unlock (&priv->mutex);
 }
 
 /**
@@ -1011,8 +1003,6 @@ gst_app_sink_set_max_buffers (GstAppSink * appsink, guint max)
  * Get the maximum amount of buffers that can be queued in @appsink.
  *
  * Returns: The maximum amount of buffers that can be queued.
- *
- * Since: 0.10.22
  */
 guint
 gst_app_sink_get_max_buffers (GstAppSink * appsink)
@@ -1024,9 +1014,9 @@ gst_app_sink_get_max_buffers (GstAppSink * appsink)
 
   priv = appsink->priv;
 
-  g_mutex_lock (priv->mutex);
+  g_mutex_lock (&priv->mutex);
   result = priv->max_buffers;
-  g_mutex_unlock (priv->mutex);
+  g_mutex_unlock (&priv->mutex);
 
   return result;
 }
@@ -1038,8 +1028,6 @@ gst_app_sink_get_max_buffers (GstAppSink * appsink)
  *
  * Instruct @appsink to drop old buffers when the maximum amount of queued
  * buffers is reached.
- *
- * Since: 0.10.22
  */
 void
 gst_app_sink_set_drop (GstAppSink * appsink, gboolean drop)
@@ -1050,13 +1038,13 @@ gst_app_sink_set_drop (GstAppSink * appsink, gboolean drop)
 
   priv = appsink->priv;
 
-  g_mutex_lock (priv->mutex);
+  g_mutex_lock (&priv->mutex);
   if (priv->drop != drop) {
     priv->drop = drop;
     /* signal the change */
-    g_cond_signal (priv->cond);
+    g_cond_signal (&priv->cond);
   }
-  g_mutex_unlock (priv->mutex);
+  g_mutex_unlock (&priv->mutex);
 }
 
 /**
@@ -1068,8 +1056,6 @@ gst_app_sink_set_drop (GstAppSink * appsink, gboolean drop)
  *
  * Returns: %TRUE if @appsink is dropping old buffers when the queue is
  * filled.
- *
- * Since: 0.10.22
  */
 gboolean
 gst_app_sink_get_drop (GstAppSink * appsink)
@@ -1081,9 +1067,9 @@ gst_app_sink_get_drop (GstAppSink * appsink)
 
   priv = appsink->priv;
 
-  g_mutex_lock (priv->mutex);
+  g_mutex_lock (&priv->mutex);
   result = priv->drop;
-  g_mutex_unlock (priv->mutex);
+  g_mutex_unlock (&priv->mutex);
 
   return result;
 }
@@ -1110,8 +1096,6 @@ gst_app_sink_get_drop (GstAppSink * appsink)
  * element is set to the READY/NULL state.
  *
  * Returns: a #GstBuffer or NULL when the appsink is stopped or EOS.
- *
- * Since: 0.10.22
  */
 GstSample *
 gst_app_sink_pull_preroll (GstAppSink * appsink)
@@ -1123,7 +1107,7 @@ gst_app_sink_pull_preroll (GstAppSink * appsink)
 
   priv = appsink->priv;
 
-  g_mutex_lock (priv->mutex);
+  g_mutex_lock (&priv->mutex);
 
   while (TRUE) {
     GST_DEBUG_OBJECT (appsink, "trying to grab a buffer");
@@ -1138,13 +1122,13 @@ gst_app_sink_pull_preroll (GstAppSink * appsink)
 
     /* nothing to return, wait */
     GST_DEBUG_OBJECT (appsink, "waiting for the preroll buffer");
-    g_cond_wait (priv->cond, priv->mutex);
+    g_cond_wait (&priv->cond, &priv->mutex);
   }
   sample =
       gst_sample_new (priv->preroll, priv->preroll_caps, &priv->last_segment,
       NULL);
   GST_DEBUG_OBJECT (appsink, "we have the preroll sample %p", sample);
-  g_mutex_unlock (priv->mutex);
+  g_mutex_unlock (&priv->mutex);
 
   return sample;
 
@@ -1152,13 +1136,13 @@ gst_app_sink_pull_preroll (GstAppSink * appsink)
 eos:
   {
     GST_DEBUG_OBJECT (appsink, "we are EOS, return NULL");
-    g_mutex_unlock (priv->mutex);
+    g_mutex_unlock (&priv->mutex);
     return NULL;
   }
 not_started:
   {
     GST_DEBUG_OBJECT (appsink, "we are stopped, return NULL");
-    g_mutex_unlock (priv->mutex);
+    g_mutex_unlock (&priv->mutex);
     return NULL;
   }
 }
@@ -1180,8 +1164,6 @@ not_started:
  * %NULL. Use gst_app_sink_is_eos () to check for the EOS condition.
  *
  * Returns: a #GstBuffer or NULL when the appsink is stopped or EOS.
- *
- * Since: 0.10.22
  */
 
 GstSample *
@@ -1195,7 +1177,7 @@ gst_app_sink_pull_sample (GstAppSink * appsink)
 
   priv = appsink->priv;
 
-  g_mutex_lock (priv->mutex);
+  g_mutex_lock (&priv->mutex);
 
   while (TRUE) {
     GST_DEBUG_OBJECT (appsink, "trying to grab a buffer");
@@ -1210,15 +1192,15 @@ gst_app_sink_pull_sample (GstAppSink * appsink)
 
     /* nothing to return, wait */
     GST_DEBUG_OBJECT (appsink, "waiting for a buffer");
-    g_cond_wait (priv->cond, priv->mutex);
+    g_cond_wait (&priv->cond, &priv->mutex);
   }
   buffer = dequeue_buffer (appsink);
   GST_DEBUG_OBJECT (appsink, "we have a buffer %p", buffer);
   sample = gst_sample_new (buffer, priv->last_caps, &priv->last_segment, NULL);
   gst_buffer_unref (buffer);
 
-  g_cond_signal (priv->cond);
-  g_mutex_unlock (priv->mutex);
+  g_cond_signal (&priv->cond);
+  g_mutex_unlock (&priv->mutex);
 
   return sample;
 
@@ -1226,19 +1208,19 @@ gst_app_sink_pull_sample (GstAppSink * appsink)
 eos:
   {
     GST_DEBUG_OBJECT (appsink, "we are EOS, return NULL");
-    g_mutex_unlock (priv->mutex);
+    g_mutex_unlock (&priv->mutex);
     return NULL;
   }
 not_started:
   {
     GST_DEBUG_OBJECT (appsink, "we are stopped, return NULL");
-    g_mutex_unlock (priv->mutex);
+    g_mutex_unlock (&priv->mutex);
     return NULL;
   }
 }
 
 /**
- * gst_app_sink_set_callbacks:
+ * gst_app_sink_set_callbacks: (skip)
  * @appsink: a #GstAppSink
  * @callbacks: the callbacks
  * @user_data: a user_data argument for the callbacks
@@ -1250,8 +1232,6 @@ not_started:
  *
  * If callbacks are installed, no signals will be emitted for performance
  * reasons.
- *
- * Since: 0.10.23
  */
 void
 gst_app_sink_set_callbacks (GstAppSink * appsink,
