@@ -257,9 +257,7 @@ gst_rtp_base_depayload_chain (GstPad * pad, GstObject * parent, GstBuffer * in)
   if (G_UNLIKELY (!priv->negotiated))
     goto not_negotiated;
 
-  /* we must validate, it's possible that this element is plugged right after a
-   * network receiver and we don't want to operate on invalid data */
-  if (G_UNLIKELY (!gst_rtp_buffer_validate (in)))
+  if (G_UNLIKELY (!gst_rtp_buffer_map (in, GST_MAP_READ, &rtp)))
     goto invalid_buffer;
 
   if (!priv->discont)
@@ -275,7 +273,6 @@ gst_rtp_base_depayload_chain (GstPad * pad, GstObject * parent, GstBuffer * in)
   priv->dts = dts;
   priv->duration = GST_BUFFER_DURATION (in);
 
-  gst_rtp_buffer_map (in, GST_MAP_READ, &rtp);
   seqnum = gst_rtp_buffer_get_seq (&rtp);
   rtptime = gst_rtp_buffer_get_timestamp (&rtp);
   gst_rtp_buffer_unmap (&rtp);
@@ -419,8 +416,8 @@ gst_rtp_base_depayload_handle_event (GstRTPBaseDepayload * filter,
       if (gst_event_has_name (event, "GstRTPPacketLost")) {
         /* we get this event from the jitterbuffer when it considers a packet as
          * being lost. We send it to our packet_lost vmethod. The default
-         * implementation will make time progress by pushing out a NEWSEGMENT
-         * update event. Subclasses can override and to one of the following:
+         * implementation will make time progress by pushing out a GAP event.
+         * Subclasses can override and to one of the following:
          *  - Adjust timestamp/duration to something more accurate before
          *    calling the parent (default) packet_lost method.
          *  - do some more advanced error concealing on the already received
@@ -464,8 +461,7 @@ gst_rtp_base_depayload_handle_sink_event (GstPad * pad, GstObject * parent,
 }
 
 static GstEvent *
-create_segment_event (GstRTPBaseDepayload * filter, gboolean update,
-    GstClockTime position)
+create_segment_event (GstRTPBaseDepayload * filter, GstClockTime position)
 {
   GstEvent *event;
   GstClockTime stop;
@@ -555,7 +551,7 @@ gst_rtp_base_depayload_prepare_push (GstRTPBaseDepayload * filter,
   if (G_UNLIKELY (filter->need_newsegment)) {
     GstEvent *event;
 
-    event = create_segment_event (filter, FALSE, 0);
+    event = create_segment_event (filter, 0);
 
     gst_pad_push_event (filter->srcpad, event);
 
@@ -603,8 +599,6 @@ gst_rtp_base_depayload_push (GstRTPBaseDepayload * filter, GstBuffer * out_buf)
  * @out_list.
  *
  * Returns: a #GstFlowReturn.
- *
- * Since: 0.10.32
  */
 GstFlowReturn
 gst_rtp_base_depayload_push_list (GstRTPBaseDepayload * filter,
@@ -622,7 +616,7 @@ gst_rtp_base_depayload_push_list (GstRTPBaseDepayload * filter,
   return res;
 }
 
-/* convert the PacketLost event form a jitterbuffer to a segment update.
+/* convert the PacketLost event form a jitterbuffer to a GAP event.
  * subclasses can override this.  */
 static gboolean
 gst_rtp_base_depayload_packet_lost (GstRTPBaseDepayload * filter,
@@ -645,8 +639,8 @@ gst_rtp_base_depayload_packet_lost (GstRTPBaseDepayload * filter,
   if (duration != -1)
     position += duration;
 
-  /* update the current segment with the elapsed time */
-  sevent = create_segment_event (filter, TRUE, position);
+  /* send GAP event */
+  sevent = gst_event_new_gap (timestamp, duration);
 
   return gst_pad_push_event (filter->srcpad, sevent);
 }
