@@ -49,6 +49,8 @@ struct _GstRTPBasePayloadPrivate
 
   gint64 prop_max_ptime;
   gint64 caps_max_ptime;
+
+  gboolean negotiated;
 };
 
 /* RTPBasePayload signals and args */
@@ -195,8 +197,6 @@ gst_rtp_base_payload_class_init (GstRTPBasePayloadClass * klass)
    * GstRTPBaseAudioPayload:min-ptime:
    *
    * Minimum duration of the packet data in ns (can't go above MTU)
-   *
-   * Since: 0.10.13
    **/
   g_object_class_install_property (G_OBJECT_CLASS (klass), PROP_MIN_PTIME,
       g_param_spec_int64 ("min-ptime", "Min packet time",
@@ -220,8 +220,6 @@ gst_rtp_base_payload_class_init (GstRTPBasePayloadClass * klass)
    * option is disabled, RTP timestamps are generated from the GStreamer
    * timestamps, which could result in RTP timestamps that don't increment with
    * the amount of data in the packet.
-   *
-   * Since: 0.10.25
    */
   g_object_class_install_property (G_OBJECT_CLASS (klass), PROP_PERFECT_RTPTIME,
       g_param_spec_boolean ("perfect-rtptime", "Perfect RTP Time",
@@ -231,8 +229,6 @@ gst_rtp_base_payload_class_init (GstRTPBasePayloadClass * klass)
    * GstRTPBaseAudioPayload:ptime-multiple:
    *
    * Force buffers to be multiples of this duration in ns (0 disables)
-   *
-   * Since: 0.10.29
    **/
   g_object_class_install_property (G_OBJECT_CLASS (klass), PROP_PTIME_MULTIPLE,
       g_param_spec_int64 ("ptime-multiple", "Packet time multiple",
@@ -364,6 +360,10 @@ gst_rtp_base_payload_sink_event_default (GstRTPBasePayload * rtpbasepayload,
       rtpbasepayload_class = GST_RTP_BASE_PAYLOAD_GET_CLASS (rtpbasepayload);
       if (rtpbasepayload_class->set_caps)
         res = rtpbasepayload_class->set_caps (rtpbasepayload, caps);
+      else
+        res = TRUE;
+
+      rtpbasepayload->priv->negotiated = res;
 
       gst_event_unref (event);
       break;
@@ -471,6 +471,9 @@ gst_rtp_base_payload_chain (GstPad * pad, GstObject * parent,
   if (!rtpbasepayload_class->handle_buffer)
     goto no_function;
 
+  if (!rtpbasepayload->priv->negotiated)
+    goto not_negotiated;
+
   ret = rtpbasepayload_class->handle_buffer (rtpbasepayload, buffer);
 
   return ret;
@@ -483,6 +486,14 @@ no_function:
     gst_buffer_unref (buffer);
     return GST_FLOW_ERROR;
   }
+not_negotiated:
+  {
+    GST_ELEMENT_ERROR (rtpbasepayload, CORE, NEGOTIATION, (NULL),
+        ("No input format was negotiated, i.e. no caps event was received. "
+            "Perhaps you need a parser or typefind element before the payloader"));
+    gst_buffer_unref (buffer);
+    return GST_FLOW_NOT_NEGOTIATED;
+  }
 }
 
 /**
@@ -753,7 +764,9 @@ set_headers (GstBuffer ** buffer, guint idx, gpointer user_data)
   HeaderData *data = user_data;
   GstRTPBuffer rtp = { NULL, };
 
-  gst_rtp_buffer_map (*buffer, GST_MAP_WRITE, &rtp);
+  if (!gst_rtp_buffer_map (*buffer, GST_MAP_WRITE, &rtp))
+    goto map_failed;
+
   gst_rtp_buffer_set_ssrc (&rtp, data->ssrc);
   gst_rtp_buffer_set_payload_type (&rtp, data->pt);
   gst_rtp_buffer_set_seq (&rtp, data->seqnum);
@@ -764,6 +777,12 @@ set_headers (GstBuffer ** buffer, guint idx, gpointer user_data)
   data->seqnum++;
 
   return TRUE;
+  /* ERRORS */
+map_failed:
+  {
+    GST_ERROR ("failed to map buffer %p", *buffer);
+    return FALSE;
+  }
 }
 
 /* Updates the SSRC, payload type, seqnum and timestamp of the RTP buffer
@@ -854,8 +873,8 @@ gst_rtp_base_payload_prepare_push (GstRTPBasePayload * payload,
       (is_list) ? -1 : gst_buffer_get_size (GST_BUFFER (obj)),
       payload->seqnum, data.rtptime, GST_TIME_ARGS (data.pts));
 
-  if (g_atomic_int_compare_and_exchange (&payload->
-          priv->notified_first_timestamp, 1, 0)) {
+  if (g_atomic_int_compare_and_exchange (&payload->priv->
+          notified_first_timestamp, 1, 0)) {
     g_object_notify (G_OBJECT (payload), "timestamp");
     g_object_notify (G_OBJECT (payload), "seqnum");
   }
@@ -882,8 +901,6 @@ no_rate:
  * This function takes ownership of @list.
  *
  * Returns: a #GstFlowReturn.
- *
- * Since: 0.10.24
  */
 GstFlowReturn
 gst_rtp_base_payload_push_list (GstRTPBasePayload * payload,
@@ -1077,6 +1094,7 @@ gst_rtp_base_payload_change_state (GstElement * element,
       rtpbasepayload->timestamp = rtpbasepayload->ts_base;
       g_atomic_int_set (&rtpbasepayload->priv->notified_first_timestamp, 1);
       priv->base_offset = GST_BUFFER_OFFSET_NONE;
+      priv->negotiated = FALSE;
       break;
     default:
       break;
