@@ -125,6 +125,8 @@ struct _GstTagDemuxPrivate
 #define TYPE_FIND_MIN_SIZE 8192
 #define TYPE_FIND_MAX_SIZE 65536
 
+#define DEFAULT_PULL_BLOCKSIZE 4096
+
 GST_DEBUG_CATEGORY_STATIC (tagdemux_debug);
 #define GST_CAT_DEFAULT (tagdemux_debug)
 
@@ -265,11 +267,11 @@ gst_tag_demux_reset (GstTagDemux * tagdemux)
   gst_caps_replace (caps_p, NULL);
 
   if (tagdemux->priv->event_tags) {
-    gst_tag_list_free (tagdemux->priv->event_tags);
+    gst_tag_list_unref (tagdemux->priv->event_tags);
     tagdemux->priv->event_tags = NULL;
   }
   if (tagdemux->priv->parsed_tags) {
-    gst_tag_list_free (tagdemux->priv->parsed_tags);
+    gst_tag_list_unref (tagdemux->priv->parsed_tags);
     tagdemux->priv->parsed_tags = NULL;
   }
 
@@ -344,12 +346,17 @@ gst_tag_demux_set_src_caps (GstTagDemux * tagdemux, GstCaps * new_caps)
   GstCaps *old_caps = tagdemux->priv->src_caps;
 
   if (old_caps == NULL || !gst_caps_is_equal (new_caps, old_caps)) {
+    gchar *stream_id = gst_pad_create_stream_id (tagdemux->priv->srcpad,
+        GST_ELEMENT_CAST (tagdemux), NULL);
 
     gst_caps_replace (&tagdemux->priv->src_caps, new_caps);
 
     GST_DEBUG_OBJECT (tagdemux, "Changing src pad caps to %" GST_PTR_FORMAT,
         tagdemux->priv->src_caps);
 
+    gst_pad_push_event (tagdemux->priv->srcpad,
+        gst_event_new_stream_start (stream_id));
+    g_free (stream_id);
     gst_pad_set_caps (tagdemux->priv->srcpad, tagdemux->priv->src_caps);
   } else {
     /* Caps never changed */
@@ -765,30 +772,30 @@ gst_tag_demux_srcpad_event (GstPad * pad, GstObject * parent, GstEvent * event)
     {
       gdouble rate;
       GstFormat format;
-      GstSeekType cur_type, stop_type;
+      GstSeekType start_type, stop_type;
       GstSeekFlags flags;
-      gint64 cur, stop;
+      gint64 start, stop;
 
       gst_event_parse_seek (event, &rate, &format, &flags,
-          &cur_type, &cur, &stop_type, &stop);
+          &start_type, &start, &stop_type, &stop);
 
       if (format == GST_FORMAT_BYTES &&
           tagdemux->priv->state == GST_TAG_DEMUX_STREAMING &&
           gst_pad_is_linked (tagdemux->priv->sinkpad)) {
         GstEvent *upstream;
 
-        switch (cur_type) {
+        switch (start_type) {
           case GST_SEEK_TYPE_SET:
-            if (cur == -1)
-              cur = 0;
-            cur += tagdemux->priv->strip_start;
+            if (start == -1)
+              start = 0;
+            start += tagdemux->priv->strip_start;
             break;
           case GST_SEEK_TYPE_END:
             /* Adjust the seek to be relative to the start of any end tag
              * (note: 10 bytes before end is represented by stop=-10) */
-            if (cur > 0)
-              cur = 0;
-            cur -= tagdemux->priv->strip_end;
+            if (start > 0)
+              start = 0;
+            start -= tagdemux->priv->strip_end;
             break;
           case GST_SEEK_TYPE_NONE:
           default:
@@ -813,7 +820,7 @@ gst_tag_demux_srcpad_event (GstPad * pad, GstObject * parent, GstEvent * event)
             break;
         }
         upstream = gst_event_new_seek (rate, format, flags,
-            cur_type, cur, stop_type, stop);
+            start_type, start, stop_type, stop);
         res = gst_pad_push_event (tagdemux->priv->sinkpad, upstream);
       }
       break;
@@ -960,7 +967,7 @@ gst_tag_demux_pull_end_tag (GstTagDemux * demux, GstTagList ** tags)
 
 done:
   if (new_tags)
-    gst_tag_list_free (new_tags);
+    gst_tag_list_unref (new_tags);
   if (buffer)
     gst_buffer_unref (buffer);
   return res;
@@ -1082,7 +1089,7 @@ gst_tag_demux_pull_start_tag (GstTagDemux * demux, GstTagList ** tags)
 
 done:
   if (new_tags)
-    gst_tag_list_free (new_tags);
+    gst_tag_list_unref (new_tags);
   if (buffer)
     gst_buffer_unref (buffer);
   return res;
@@ -1131,9 +1138,9 @@ gst_tag_demux_element_find (GstTagDemux * demux)
   }
 
   if (start_tags)
-    gst_tag_list_free (start_tags);
+    gst_tag_list_unref (start_tags);
   if (end_tags)
-    gst_tag_list_free (end_tags);
+    gst_tag_list_unref (end_tags);
 
   if (!e_tag_ok && !s_tag_ok)
     goto no_tags;
@@ -1146,14 +1153,17 @@ gst_tag_demux_element_find (GstTagDemux * demux)
       demux->priv->strip_start + demux->priv->strip_end)
     goto no_data;
 
-  /* 2 - Do typefinding on data */
+  /* 2 - Do typefinding on data, but not if downstream is in charge */
+  if (GST_PAD_MODE (demux->priv->srcpad) == GST_PAD_MODE_PULL)
+    goto skip_typefinding;
+
   caps = gst_type_find_helper_get_range (GST_OBJECT (demux), NULL,
       (GstTypeFindHelperGetRangeFunction) gst_tag_demux_read_range,
       demux->priv->upstream_size
       - (demux->priv->strip_start + demux->priv->strip_end), NULL,
       &probability);
 
-  GST_DEBUG_OBJECT (demux, "Found type %" GST_PTR_FORMAT " with a "
+  GST_INFO_OBJECT (demux, "Found type %" GST_PTR_FORMAT " with a "
       "probability of %u", caps, probability);
 
   /* 3 - If we didn't find the caps, fail */
@@ -1168,6 +1178,11 @@ gst_tag_demux_element_find (GstTagDemux * demux)
   gst_tag_demux_set_src_caps (demux, caps);
   gst_caps_unref (caps);
 
+skip_typefinding:
+
+  /* set it again, in case we skipped typefinding */
+  demux->priv->state = GST_TAG_DEMUX_STREAMING;
+
   return ret;
 
   /* ERRORS */
@@ -1217,8 +1232,33 @@ gst_tag_demux_element_loop (GstTagDemux * demux)
       ret = gst_tag_demux_element_find (demux);
       break;
     case GST_TAG_DEMUX_STREAMING:
-      ret = GST_FLOW_ERROR;
+    {
+      GstBuffer *outbuf = NULL;
+
+      if (demux->priv->need_newseg) {
+        demux->priv->need_newseg = FALSE;
+        /* FIXME: check segment, should be 0-N for downstream */
+        gst_tag_demux_send_new_segment (demux);
+      }
+
+      /* Send our own pending tag event */
+      if (demux->priv->send_tag_event) {
+        gst_tag_demux_send_tag_event (demux);
+        demux->priv->send_tag_event = FALSE;
+      }
+
+      /* Pull data and push it downstream */
+      ret = gst_pad_pull_range (demux->priv->sinkpad, demux->priv->offset,
+          DEFAULT_PULL_BLOCKSIZE, &outbuf);
+
+      if (ret != GST_FLOW_OK)
+        break;
+
+      demux->priv->offset += gst_buffer_get_size (outbuf);
+
+      ret = gst_pad_push (demux->priv->srcpad, outbuf);
       break;
+    }
     default:
       ret = GST_FLOW_ERROR;
       break;
@@ -1252,6 +1292,8 @@ pause:
         gst_element_post_message (GST_ELEMENT_CAST (demux),
             gst_message_new_segment_done (GST_OBJECT_CAST (demux),
                 GST_FORMAT_BYTES, stop));
+        gst_pad_push_event (demux->priv->srcpad,
+            gst_event_new_segment_done (GST_FORMAT_BYTES, stop));
       } else {
         push_eos = TRUE;
       }
@@ -1287,6 +1329,10 @@ gst_tag_demux_sink_activate_mode (GstPad * pad, GstObject * parent,
       res = TRUE;
       break;
   }
+
+  if (active)
+    GST_TAG_DEMUX (parent)->priv->state = GST_TAG_DEMUX_READ_START_TAG;
+
   return res;
 }
 
@@ -1312,7 +1358,8 @@ gst_tag_demux_sink_activate (GstPad * sinkpad, GstObject * parent)
     goto activate_push;
   }
 
-  pull_mode = gst_query_has_scheduling_mode (query, GST_PAD_MODE_PULL);
+  pull_mode = gst_query_has_scheduling_mode_with_flags (query,
+      GST_PAD_MODE_PULL, GST_SCHEDULING_FLAG_SEEKABLE);
   gst_query_unref (query);
 
   if (!pull_mode)
@@ -1323,13 +1370,12 @@ gst_tag_demux_sink_activate (GstPad * sinkpad, GstObject * parent)
 
   /* only start our task if we ourselves decide to start in pull mode */
   return gst_pad_start_task (sinkpad,
-      (GstTaskFunction) gst_tag_demux_element_loop, demux);
+      (GstTaskFunction) gst_tag_demux_element_loop, demux, NULL);
 
 activate_push:
   {
     GST_DEBUG_OBJECT (demux, "No pull mode. Changing to push, but won't be "
         "able to read end tags");
-    demux->priv->state = GST_TAG_DEMUX_READ_START_TAG;
     return gst_pad_activate_mode (sinkpad, GST_PAD_MODE_PUSH, TRUE);
   }
 }
@@ -1454,6 +1500,7 @@ gst_tag_demux_pad_query (GstPad * pad, GstObject * parent, GstQuery * query)
   gint64 result;
   gboolean res = TRUE;
 
+  /* FIXME: locking ? */
   switch (GST_QUERY_TYPE (query)) {
     case GST_QUERY_SCHEDULING:
       res = gst_pad_peer_query (demux->priv->sinkpad, query);
@@ -1477,7 +1524,20 @@ gst_tag_demux_pad_query (GstPad * pad, GstObject * parent, GstQuery * query)
 
       gst_query_parse_duration (query, &format, &result);
       if (format == GST_FORMAT_BYTES) {
+        /* if downstream activated us in pull mode right away, e.g. in case of
+         * filesrc ! id3demux ! xyzparse ! .., read tags here, since we don't
+         * have a streaming thread of our own to do that. We do it here and
+         * not in get_range(), so we can return the right size in bytes.. */
+        if (demux->priv->state == GST_TAG_DEMUX_READ_START_TAG &&
+            GST_PAD_MODE (demux->priv->srcpad) == GST_PAD_MODE_PULL) {
+          GstFlowReturn flow G_GNUC_UNUSED;
+
+          flow = gst_tag_demux_element_find (demux);
+          GST_INFO_OBJECT (demux, "pulled tags: %s", gst_flow_get_name (flow));
+        }
         result -= demux->priv->strip_start + demux->priv->strip_end;
+        if (result < 0)
+          result = 0;
         gst_query_set_duration (query, format, result);
       }
       break;
@@ -1516,13 +1576,8 @@ gst_tag_demux_send_tag_event (GstTagDemux * demux)
   GstTagList *merged = gst_tag_list_merge (demux->priv->event_tags,
       demux->priv->parsed_tags, GST_TAG_MERGE_KEEP);
 
-  if (demux->priv->parsed_tags)
-    gst_element_post_message (GST_ELEMENT (demux),
-        gst_message_new_tag (GST_OBJECT (demux),
-            gst_tag_list_copy (demux->priv->parsed_tags)));
-
   if (merged) {
-    GstEvent *event = gst_event_new_tag ("GstTagDemuxer", merged);
+    GstEvent *event = gst_event_new_tag (merged);
 
     GST_EVENT_TIMESTAMP (event) = 0;
     GST_DEBUG_OBJECT (demux, "Sending tag event on src pad");
