@@ -201,10 +201,13 @@ typedef enum
  *       Typical usage in decoders is to set this on the opaque value provided
  *       to the library and get back the frame using gst_video_decoder_get_frame()
  * @distance_from_sync: Distance in frames from the last synchronization point.
- * @input_buffer: the input #GstBuffer that created this frame.
+ * @input_buffer: the input #GstBuffer that created this frame. The buffer is owned
+ *           by the frame and references to the frame instead of the buffer should
  * @output_buffer: the output #GstBuffer. Implementations should set this either
  *           directly, or by using the @gst_video_decoder_alloc_output_frame() or
- *           @gst_video_decoder_alloc_output_buffer() methods.
+ *           @gst_video_decoder_alloc_output_buffer() methods. The buffer is owned
+ *           by the frame and references to the frame instead of the buffer should
+ *           be kept.
  * @deadline: Running time when the frame will be used.
  * @events: Events that will be pushed downstream before this frame is pushed.
  *
@@ -219,9 +222,9 @@ struct _GstVideoCodecFrame
   guint32 flags;
 
   /*< public >*/
-  gint system_frame_number;	/* ED */
-  gint decode_frame_number;	/* ED */
-  gint presentation_frame_number; /* ED */
+  guint32 system_frame_number;	/* ED */
+  guint32 decode_frame_number;	/* ED */
+  guint32 presentation_frame_number; /* ED */
 
   GstClockTime dts;       /* ED */
   GstClockTime pts;       /* ED */
@@ -243,7 +246,12 @@ struct _GstVideoCodecFrame
   gpointer       user_data;
   GDestroyNotify user_data_destroy_notify;
 
-  void         *padding[GST_PADDING_LARGE];
+  union {
+    struct {
+      GstClockTime ts;
+    } ABI;
+    void         *padding[GST_PADDING_LARGE];
+  } abidata;
 };
 
 /* GstVideoCodecState */
