@@ -26,7 +26,6 @@
 
 #include <gst/pbutils/pbutils.h>
 #include <gst/gst-i18n-plugin.h>
-#include "gst/glib-compat-private.h"
 
 GST_DEBUG_CATEGORY_STATIC (gst_play_sink_convert_bin_debug);
 #define GST_CAT_DEFAULT gst_play_sink_convert_bin_debug
@@ -60,11 +59,11 @@ is_raw_caps (GstCaps * caps, gboolean audio)
   for (i = 0; i < n; i++) {
     s = gst_caps_get_structure (caps, i);
     name = gst_structure_get_name (s);
-    if (!g_str_equal (name, prefix))
-      return FALSE;
+    if (g_str_equal (name, prefix))
+      return TRUE;
   }
 
-  return TRUE;
+  return FALSE;
 }
 
 static void
@@ -272,7 +271,7 @@ gst_play_sink_convert_bin_sink_event (GstPad * pad, GstObject * parent,
       break;
   }
 
-  ret = gst_proxy_pad_event_default (pad, parent, gst_event_ref (event));
+  ret = gst_pad_event_default (pad, parent, gst_event_ref (event));
 
   if (GST_EVENT_TYPE (event) == GST_EVENT_SEGMENT) {
     GstSegment seg;
@@ -464,7 +463,7 @@ gst_play_sink_convert_bin_query (GstPad * pad, GstObject * parent,
       break;
     }
     default:
-      res = gst_proxy_pad_query_default (pad, parent, query);
+      res = gst_pad_query_default (pad, parent, query);
       break;
   }
   return res;
@@ -501,7 +500,7 @@ gst_play_sink_convert_bin_finalize (GObject * object)
   GstPlaySinkConvertBin *self = GST_PLAY_SINK_CONVERT_BIN_CAST (object);
 
   gst_object_unref (self->sink_proxypad);
-  g_mutex_free (self->lock);
+  g_mutex_clear (&self->lock);
 
   G_OBJECT_CLASS (parent_class)->finalize (object);
 }
@@ -617,7 +616,7 @@ gst_play_sink_convert_bin_init (GstPlaySinkConvertBin * self)
 {
   GstPadTemplate *templ;
 
-  self->lock = g_mutex_new ();
+  g_mutex_init (&self->lock);
   gst_segment_init (&self->segment, GST_FORMAT_UNDEFINED);
 
   templ = gst_static_pad_template_get (&sinktemplate);
