@@ -22,7 +22,6 @@
 #endif
 
 #include "gststreamsynchronizer.h"
-#include "gst/glib-compat-private.h"
 
 GST_DEBUG_CATEGORY_STATIC (stream_synchronizer_debug);
 #define GST_CAT_DEFAULT stream_synchronizer_debug
@@ -31,7 +30,7 @@ GST_DEBUG_CATEGORY_STATIC (stream_synchronizer_debug);
     GST_LOG_OBJECT (obj,                                                \
                     "locking from thread %p",                           \
                     g_thread_self ());                                  \
-    g_mutex_lock (GST_STREAM_SYNCHRONIZER_CAST(obj)->lock);                \
+    g_mutex_lock (&GST_STREAM_SYNCHRONIZER_CAST(obj)->lock);                \
     GST_LOG_OBJECT (obj,                                                \
                     "locked from thread %p",                            \
                     g_thread_self ());                                  \
@@ -41,7 +40,7 @@ GST_DEBUG_CATEGORY_STATIC (stream_synchronizer_debug);
     GST_LOG_OBJECT (obj,                                                \
                     "unlocking from thread %p",                         \
                     g_thread_self ());                                  \
-    g_mutex_unlock (GST_STREAM_SYNCHRONIZER_CAST(obj)->lock);              \
+    g_mutex_unlock (&GST_STREAM_SYNCHRONIZER_CAST(obj)->lock);              \
 } G_STMT_END
 
 static GstStaticPadTemplate srctemplate = GST_STATIC_PAD_TEMPLATE ("src_%u",
@@ -53,8 +52,6 @@ static GstStaticPadTemplate sinktemplate = GST_STATIC_PAD_TEMPLATE ("sink_%u",
     GST_PAD_REQUEST,
     GST_STATIC_CAPS_ANY);
 
-static const gboolean passthrough = TRUE;
-
 #define gst_stream_synchronizer_parent_class parent_class
 G_DEFINE_TYPE (GstStreamSynchronizer, gst_stream_synchronizer,
     GST_TYPE_ELEMENT);
@@ -67,40 +64,38 @@ typedef struct
   GstPad *sinkpad;
   GstSegment segment;
 
-  gboolean wait;
+  gboolean wait;                /* TRUE if waiting/blocking */
   gboolean new_stream;
   gboolean drop_discont;
-  gboolean is_eos;
+  gboolean is_eos;              /* TRUE if EOS was received */
   gboolean seen_data;
 
-  gint64 running_time_diff;
+  GCond stream_finish_cond;
+
+  /* seqnum of the previously received STREAM_START
+   * default: G_MAXUINT32 */
+  guint32 stream_start_seqnum;
+  guint32 segment_seqnum;
 } GstStream;
 
 /* Must be called with lock! */
-static GstPad *
+static inline GstPad *
 gst_stream_get_other_pad (GstStream * stream, GstPad * pad)
 {
   if (stream->sinkpad == pad)
     return gst_object_ref (stream->srcpad);
-  else if (stream->srcpad == pad)
+  if (stream->srcpad == pad)
     return gst_object_ref (stream->sinkpad);
 
   return NULL;
 }
 
 static GstPad *
-gst_stream_get_other_pad_from_pad (GstPad * pad)
+gst_stream_get_other_pad_from_pad (GstStreamSynchronizer * self, GstPad * pad)
 {
-  GstObject *parent = gst_pad_get_parent (pad);
-  GstStreamSynchronizer *self;
   GstStream *stream;
   GstPad *opad = NULL;
 
-  /* released pad does not have parent anymore */
-  if (!G_LIKELY (parent))
-    goto exit;
-
-  self = GST_STREAM_SYNCHRONIZER (parent);
   GST_STREAM_SYNCHRONIZER_LOCK (self);
   stream = gst_pad_get_element_private (pad);
   if (!stream)
@@ -110,9 +105,7 @@ gst_stream_get_other_pad_from_pad (GstPad * pad)
 
 out:
   GST_STREAM_SYNCHRONIZER_UNLOCK (self);
-  gst_object_unref (self);
 
-exit:
   if (!opad)
     GST_WARNING_OBJECT (pad, "Trying to get other pad after releasing");
 
@@ -127,7 +120,8 @@ gst_stream_synchronizer_iterate_internal_links (GstPad * pad,
   GstIterator *it = NULL;
   GstPad *opad;
 
-  opad = gst_stream_get_other_pad_from_pad (pad);
+  opad =
+      gst_stream_get_other_pad_from_pad (GST_STREAM_SYNCHRONIZER (parent), pad);
   if (opad) {
     GValue value = { 0, };
 
@@ -150,7 +144,8 @@ gst_stream_synchronizer_query (GstPad * pad, GstObject * parent,
 
   GST_LOG_OBJECT (pad, "Handling query %s", GST_QUERY_TYPE_NAME (query));
 
-  opad = gst_stream_get_other_pad_from_pad (pad);
+  opad =
+      gst_stream_get_other_pad_from_pad (GST_STREAM_SYNCHRONIZER (parent), pad);
   if (opad) {
     ret = gst_pad_peer_query (opad, query);
     gst_object_unref (opad);
@@ -168,9 +163,6 @@ gst_stream_synchronizer_src_event (GstPad * pad, GstObject * parent,
   GstPad *opad;
   gboolean ret = FALSE;
 
-  if (passthrough)
-    goto skip_adjustments;
-
   GST_LOG_OBJECT (pad, "Handling event %s: %" GST_PTR_FORMAT,
       GST_EVENT_TYPE_NAME (event), event);
 
@@ -179,7 +171,7 @@ gst_stream_synchronizer_src_event (GstPad * pad, GstObject * parent,
       gdouble proportion;
       GstClockTimeDiff diff;
       GstClockTime timestamp;
-      gint64 running_time_diff;
+      gint64 running_time_diff = -1;
       GstStream *stream;
 
       gst_event_parse_qos (event, NULL, &proportion, &diff, &timestamp);
@@ -188,15 +180,14 @@ gst_stream_synchronizer_src_event (GstPad * pad, GstObject * parent,
       GST_STREAM_SYNCHRONIZER_LOCK (self);
       stream = gst_pad_get_element_private (pad);
       if (stream)
-        running_time_diff = stream->running_time_diff;
-      else
-        running_time_diff = -1;
+        running_time_diff = stream->segment.base;
       GST_STREAM_SYNCHRONIZER_UNLOCK (self);
 
       if (running_time_diff == -1) {
         GST_WARNING_OBJECT (pad, "QOS event before group start");
         goto out;
-      } else if (timestamp < running_time_diff) {
+      }
+      if (timestamp < running_time_diff) {
         GST_DEBUG_OBJECT (pad, "QOS event from previous group");
         goto out;
       }
@@ -225,9 +216,7 @@ gst_stream_synchronizer_src_event (GstPad * pad, GstObject * parent,
       break;
   }
 
-skip_adjustments:
-
-  opad = gst_stream_get_other_pad_from_pad (pad);
+  opad = gst_stream_get_other_pad_from_pad (self, pad);
   if (opad) {
     ret = gst_pad_push_event (opad, event);
     gst_object_unref (opad);
@@ -246,51 +235,47 @@ gst_stream_synchronizer_sink_event (GstPad * pad, GstObject * parent,
   GstPad *opad;
   gboolean ret = FALSE;
 
-  if (passthrough)
-    goto skip_adjustments;
-
   GST_LOG_OBJECT (pad, "Handling event %s: %" GST_PTR_FORMAT,
       GST_EVENT_TYPE_NAME (event), event);
 
   switch (GST_EVENT_TYPE (event)) {
-    case GST_EVENT_SINK_MESSAGE:{
-      GstMessage *message;
+    case GST_EVENT_STREAM_START:
+    {
+      GstStream *stream;
+      guint32 seqnum = gst_event_get_seqnum (event);
+      GList *l;
+      gboolean all_wait = TRUE;
 
-      gst_event_parse_sink_message (event, &message);
-      if (gst_message_has_name (message, "playbin-stream-changed")) {
-        GstStream *stream;
+      GST_STREAM_SYNCHRONIZER_LOCK (self);
+      stream = gst_pad_get_element_private (pad);
+      if (stream && stream->stream_start_seqnum != seqnum) {
 
-        GST_STREAM_SYNCHRONIZER_LOCK (self);
-        stream = gst_pad_get_element_private (pad);
-        if (stream) {
-          GList *l;
-          gboolean all_wait = TRUE;
+        GST_DEBUG_OBJECT (pad, "Stream %d changed", stream->stream_number);
 
-          GST_DEBUG_OBJECT (pad, "Stream %d changed", stream->stream_number);
+        stream->is_eos = FALSE;
+        stream->wait = TRUE;
+        stream->new_stream = TRUE;
 
-          stream->is_eos = FALSE;
-          stream->wait = TRUE;
-          stream->new_stream = TRUE;
+        for (l = self->streams; l; l = l->next) {
+          GstStream *ostream = l->data;
+
+          all_wait = all_wait && ostream->wait;
+          if (!all_wait)
+            break;
+        }
+        if (all_wait) {
+          gint64 position = 0;
+
+          GST_DEBUG_OBJECT (self, "All streams have changed -- unblocking");
 
           for (l = self->streams; l; l = l->next) {
             GstStream *ostream = l->data;
+            gint64 stop_running_time;
+            gint64 position_running_time;
 
-            all_wait = all_wait && ostream->wait;
-            if (!all_wait)
-              break;
-          }
-          if (all_wait) {
-            gint64 position = 0;
-
-            GST_DEBUG_OBJECT (self, "All streams have changed -- unblocking");
-
-            for (l = self->streams; l; l = l->next) {
-              GstStream *ostream = l->data;
-              gint64 stop_running_time;
-              gint64 position_running_time;
-
-              ostream->wait = FALSE;
+            ostream->wait = FALSE;
 
+            if (ostream->segment.format == GST_FORMAT_TIME) {
               stop_running_time =
                   gst_segment_to_running_time (&ostream->segment,
                   GST_FORMAT_TIME, ostream->segment.stop);
@@ -301,20 +286,23 @@ gst_stream_synchronizer_sink_event (GstPad * pad, GstObject * parent,
                   MAX (position, MAX (stop_running_time,
                       position_running_time));
             }
-            position = MAX (0, position);
-            self->group_start_time = MAX (self->group_start_time, position);
+          }
+          position = MAX (0, position);
+          self->group_start_time = MAX (self->group_start_time, position);
 
-            GST_DEBUG_OBJECT (self, "New group start time: %" GST_TIME_FORMAT,
-                GST_TIME_ARGS (self->group_start_time));
+          GST_DEBUG_OBJECT (self, "New group start time: %" GST_TIME_FORMAT,
+              GST_TIME_ARGS (self->group_start_time));
 
-            g_cond_broadcast (self->stream_finish_cond);
+          for (l = self->streams; l; l = l->next) {
+            GstStream *ostream = l->data;
+            g_cond_broadcast (&ostream->stream_finish_cond);
           }
         }
-        GST_STREAM_SYNCHRONIZER_UNLOCK (self);
-      }
-      gst_message_unref (message);
-      break;
+      } else
+        GST_DEBUG_OBJECT (self, "No stream or STREAM_START from same source");
+      GST_STREAM_SYNCHRONIZER_UNLOCK (self);
     }
+      break;
     case GST_EVENT_SEGMENT:{
       GstStream *stream;
       GstSegment segment;
@@ -326,7 +314,7 @@ gst_stream_synchronizer_sink_event (GstPad * pad, GstObject * parent,
       if (stream) {
         if (stream->wait) {
           GST_DEBUG_OBJECT (pad, "Stream %d is waiting", stream->stream_number);
-          g_cond_wait (self->stream_finish_cond, self->lock);
+          g_cond_wait (&stream->stream_finish_cond, &self->lock);
           stream = gst_pad_get_element_private (pad);
           if (stream)
             stream->wait = FALSE;
@@ -341,49 +329,9 @@ gst_stream_synchronizer_sink_event (GstPad * pad, GstObject * parent,
 
       if (stream && segment.format == GST_FORMAT_TIME) {
         if (stream->new_stream) {
-          gint64 position_running_time = 0;
-          gint64 stop_running_time = 0;
-
-          if (stream->segment.format == GST_FORMAT_TIME) {
-            position_running_time =
-                gst_segment_to_running_time (&stream->segment, GST_FORMAT_TIME,
-                stream->segment.position);
-            position_running_time = MAX (position_running_time, 0);
-            stop_running_time =
-                gst_segment_to_running_time (&stream->segment, GST_FORMAT_TIME,
-                stream->segment.stop);
-            stop_running_time = MAX (position_running_time, 0);
-
-            if (stop_running_time != position_running_time) {
-              GST_WARNING_OBJECT (pad,
-                  "Gap between position and segment stop: %" GST_TIME_FORMAT
-                  " != %" GST_TIME_FORMAT, GST_TIME_ARGS (stop_running_time),
-                  GST_TIME_ARGS (position_running_time));
-            }
-
-            if (stop_running_time < position_running_time) {
-              GST_DEBUG_OBJECT (pad, "Updating stop position");
-              stream->segment.stop = stream->segment.position;
-              gst_pad_push_event (stream->srcpad,
-                  gst_event_new_segment (&stream->segment));
-            }
-            stop_running_time = MAX (stop_running_time, position_running_time);
-            GST_DEBUG_OBJECT (pad,
-                "Stop running time of last group: %" GST_TIME_FORMAT,
-                GST_TIME_ARGS (stop_running_time));
-          }
           stream->new_stream = FALSE;
           stream->drop_discont = TRUE;
-
-          if (stop_running_time < self->group_start_time) {
-            gint64 diff = self->group_start_time - stop_running_time;
-
-            GST_DEBUG_OBJECT (pad,
-                "Advancing running time for other streams by: %"
-                GST_TIME_FORMAT, GST_TIME_ARGS (diff));
-
-            segment.base += diff;
-          }
+          segment.base = self->group_start_time;
         }
 
         GST_DEBUG_OBJECT (pad, "Segment was: %" GST_SEGMENT_FORMAT,
@@ -391,14 +339,38 @@ gst_stream_synchronizer_sink_event (GstPad * pad, GstObject * parent,
         gst_segment_copy_into (&segment, &stream->segment);
         GST_DEBUG_OBJECT (pad, "Segment now is: %" GST_SEGMENT_FORMAT,
             &stream->segment);
+        stream->segment_seqnum = gst_event_get_seqnum (event);
 
         GST_DEBUG_OBJECT (pad, "Stream start running time: %" GST_TIME_FORMAT,
             GST_TIME_ARGS (stream->segment.base));
-        stream->running_time_diff = stream->segment.base;
+        {
+          GstEvent *tmpev;
+
+          tmpev = gst_event_new_segment (&stream->segment);
+          gst_event_set_seqnum (tmpev, stream->segment_seqnum);
+          gst_event_unref (event);
+          event = tmpev;
+        }
+
       } else if (stream) {
         GST_WARNING_OBJECT (pad, "Non-TIME segment: %s",
             gst_format_get_name (segment.format));
         gst_segment_init (&stream->segment, GST_FORMAT_UNDEFINED);
+        /* Since this stream is not time-based, we mark it so that
+         * other streams don't wait forever on it */
+        stream->wait = TRUE;
+      }
+      GST_STREAM_SYNCHRONIZER_UNLOCK (self);
+      break;
+    }
+    case GST_EVENT_FLUSH_START:{
+      GstStream *stream;
+
+      GST_STREAM_SYNCHRONIZER_LOCK (self);
+      stream = gst_pad_get_element_private (pad);
+      if (stream) {
+        GST_DEBUG_OBJECT (pad, "Flushing streams");
+        g_cond_broadcast (&stream->stream_finish_cond);
       }
       GST_STREAM_SYNCHRONIZER_UNLOCK (self);
       break;
@@ -418,6 +390,7 @@ gst_stream_synchronizer_sink_event (GstPad * pad, GstObject * parent,
         stream->new_stream = FALSE;
         stream->drop_discont = FALSE;
         stream->seen_data = FALSE;
+        g_cond_broadcast (&stream->stream_finish_cond);
       }
       GST_STREAM_SYNCHRONIZER_UNLOCK (self);
       break;
@@ -481,22 +454,21 @@ gst_stream_synchronizer_sink_event (GstPad * pad, GstObject * parent,
         /* if EOS, but no data has passed, then send something to replace EOS
          * for preroll purposes */
         if (!seen_data) {
-          GstBuffer *buf = gst_buffer_new ();
+          GstEvent *gap_event;
 
-          gst_pad_push (srcpad, buf);
+          gap_event = gst_event_new_gap (0, 0);
+          gst_pad_push_event (srcpad, gap_event);
         }
       }
       gst_object_unref (srcpad);
+      gst_event_unref (event);
       goto done;
-      break;
     }
     default:
       break;
   }
 
-skip_adjustments:
-
-  opad = gst_stream_get_other_pad_from_pad (pad);
+  opad = gst_stream_get_other_pad_from_pad (self, pad);
   if (opad) {
     ret = gst_pad_push_event (opad, event);
     gst_object_unref (opad);
@@ -518,15 +490,6 @@ gst_stream_synchronizer_sink_chain (GstPad * pad, GstObject * parent,
   GstClockTime timestamp = GST_CLOCK_TIME_NONE;
   GstClockTime timestamp_end = GST_CLOCK_TIME_NONE;
 
-  if (passthrough) {
-    opad = gst_stream_get_other_pad_from_pad (pad);
-    if (opad) {
-      ret = gst_pad_push (opad, buffer);
-      gst_object_unref (opad);
-    }
-    goto done;
-  }
-
   GST_LOG_OBJECT (pad, "Handling buffer %p: size=%" G_GSIZE_FORMAT
       ", timestamp=%" GST_TIME_FORMAT " duration=%" GST_TIME_FORMAT
       " offset=%" G_GUINT64_FORMAT " offset_end=%" G_GUINT64_FORMAT,
@@ -543,24 +506,28 @@ gst_stream_synchronizer_sink_chain (GstPad * pad, GstObject * parent,
   GST_STREAM_SYNCHRONIZER_LOCK (self);
   stream = gst_pad_get_element_private (pad);
 
-  if (stream)
+  if (stream) {
     stream->seen_data = TRUE;
-  if (stream && stream->drop_discont) {
-    buffer = gst_buffer_make_writable (buffer);
-    GST_BUFFER_FLAG_UNSET (buffer, GST_BUFFER_FLAG_DISCONT);
-    stream->drop_discont = FALSE;
-  }
+    if (stream->drop_discont) {
+      if (GST_BUFFER_IS_DISCONT (buffer)) {
+        GST_DEBUG_OBJECT (pad, "removing DISCONT from buffer %p", buffer);
+        buffer = gst_buffer_make_writable (buffer);
+        GST_BUFFER_FLAG_UNSET (buffer, GST_BUFFER_FLAG_DISCONT);
+      }
+      stream->drop_discont = FALSE;
+    }
 
-  if (stream && stream->segment.format == GST_FORMAT_TIME
-      && GST_CLOCK_TIME_IS_VALID (timestamp)) {
-    GST_LOG_OBJECT (pad,
-        "Updating last-stop from %" GST_TIME_FORMAT " to %" GST_TIME_FORMAT,
-        GST_TIME_ARGS (stream->segment.position), GST_TIME_ARGS (timestamp));
-    stream->segment.position = timestamp;
+    if (stream->segment.format == GST_FORMAT_TIME
+        && GST_CLOCK_TIME_IS_VALID (timestamp)) {
+      GST_LOG_OBJECT (pad,
+          "Updating position from %" GST_TIME_FORMAT " to %" GST_TIME_FORMAT,
+          GST_TIME_ARGS (stream->segment.position), GST_TIME_ARGS (timestamp));
+      stream->segment.position = timestamp;
+    }
   }
   GST_STREAM_SYNCHRONIZER_UNLOCK (self);
 
-  opad = gst_stream_get_other_pad_from_pad (pad);
+  opad = gst_stream_get_other_pad_from_pad (self, pad);
   if (opad) {
     ret = gst_pad_push (opad, buffer);
     gst_object_unref (opad);
@@ -575,7 +542,7 @@ gst_stream_synchronizer_sink_chain (GstPad * pad, GstObject * parent,
     if (stream && stream->segment.format == GST_FORMAT_TIME
         && GST_CLOCK_TIME_IS_VALID (timestamp_end)) {
       GST_LOG_OBJECT (pad,
-          "Updating last-stop from %" GST_TIME_FORMAT " to %" GST_TIME_FORMAT,
+          "Updating position from %" GST_TIME_FORMAT " to %" GST_TIME_FORMAT,
           GST_TIME_ARGS (stream->segment.position),
           GST_TIME_ARGS (timestamp_end));
       stream->segment.position = timestamp_end;
@@ -622,7 +589,6 @@ gst_stream_synchronizer_sink_chain (GstPad * pad, GstObject * parent,
     GST_STREAM_SYNCHRONIZER_UNLOCK (self);
   }
 
-done:
   return ret;
 }
 
@@ -642,6 +608,9 @@ gst_stream_synchronizer_request_new_pad (GstElement * element,
   stream = g_slice_new0 (GstStream);
   stream->transform = self;
   stream->stream_number = self->current_stream_number;
+  g_cond_init (&stream->stream_finish_cond);
+  stream->stream_start_seqnum = G_MAXUINT32;
+  stream->segment_seqnum = G_MAXUINT32;
 
   tmp = g_strdup_printf ("sink_%u", self->current_stream_number);
   stream->sinkpad = gst_pad_new_from_static_template (&sinktemplate, tmp);
@@ -727,13 +696,17 @@ gst_stream_synchronizer_release_stream (GstStreamSynchronizer * self,
         stream->segment.position);
     stop_running_time = MAX (stop_running_time, position_running_time);
 
-    GST_DEBUG_OBJECT (stream->sinkpad,
-        "Stop running time was: %" GST_TIME_FORMAT,
-        GST_TIME_ARGS (stop_running_time));
+    if (stop_running_time > self->group_start_time) {
+      GST_DEBUG_OBJECT (stream->sinkpad,
+          "Updating global start running time from %" GST_TIME_FORMAT " to %"
+          GST_TIME_FORMAT, GST_TIME_ARGS (self->group_start_time),
+          GST_TIME_ARGS (stop_running_time));
 
-    self->group_start_time = MAX (self->group_start_time, stop_running_time);
+      self->group_start_time = stop_running_time;
+    }
   }
 
+  g_cond_clear (&stream->stream_finish_cond);
   g_slice_free (GstStream, stream);
 
   /* NOTE: In theory we have to check here if all streams
@@ -770,7 +743,7 @@ gst_stream_synchronizer_change_state (GstElement * element,
     GstStateChange transition)
 {
   GstStreamSynchronizer *self = GST_STREAM_SYNCHRONIZER (element);
-  GstStateChangeReturn ret = GST_STATE_CHANGE_SUCCESS;
+  GstStateChangeReturn ret;
 
   switch (transition) {
     case GST_STATE_CHANGE_NULL_TO_READY:
@@ -782,33 +755,29 @@ gst_stream_synchronizer_change_state (GstElement * element,
       self->group_start_time = 0;
       self->shutdown = FALSE;
       break;
-    case GST_STATE_CHANGE_PAUSED_TO_PLAYING:
-      GST_DEBUG_OBJECT (self, "State change PAUSED->PLAYING");
-      break;
-    case GST_STATE_CHANGE_PAUSED_TO_READY:
+    case GST_STATE_CHANGE_PAUSED_TO_READY:{
+      GList *l;
+
       GST_DEBUG_OBJECT (self, "State change READY->NULL");
 
       GST_STREAM_SYNCHRONIZER_LOCK (self);
-      g_cond_broadcast (self->stream_finish_cond);
+      for (l = self->streams; l; l = l->next) {
+        GstStream *ostream = l->data;
+        g_cond_broadcast (&ostream->stream_finish_cond);
+      }
       self->shutdown = TRUE;
       GST_STREAM_SYNCHRONIZER_UNLOCK (self);
+    }
     default:
       break;
   }
 
-  {
-    GstStateChangeReturn bret;
-
-    bret = GST_ELEMENT_CLASS (parent_class)->change_state (element, transition);
-    GST_DEBUG_OBJECT (self, "Base class state changed returned: %d", bret);
-    if (G_UNLIKELY (bret == GST_STATE_CHANGE_FAILURE))
-      return ret;
-  }
+  ret = GST_ELEMENT_CLASS (parent_class)->change_state (element, transition);
+  GST_DEBUG_OBJECT (self, "Base class state changed returned: %d", ret);
+  if (G_UNLIKELY (ret != GST_STATE_CHANGE_SUCCESS))
+    return ret;
 
   switch (transition) {
-    case GST_STATE_CHANGE_PLAYING_TO_PAUSED:
-      GST_DEBUG_OBJECT (self, "State change PLAYING->PAUSED");
-      break;
     case GST_STATE_CHANGE_PAUSED_TO_READY:{
       GList *l;
 
@@ -832,8 +801,6 @@ gst_stream_synchronizer_change_state (GstElement * element,
       GST_DEBUG_OBJECT (self, "State change READY->NULL");
 
       GST_STREAM_SYNCHRONIZER_LOCK (self);
-      while (self->streams)
-        gst_stream_synchronizer_release_stream (self, self->streams->data);
       self->current_stream_number = 0;
       GST_STREAM_SYNCHRONIZER_UNLOCK (self);
       break;
@@ -851,15 +818,7 @@ gst_stream_synchronizer_finalize (GObject * object)
 {
   GstStreamSynchronizer *self = GST_STREAM_SYNCHRONIZER (object);
 
-  if (self->lock) {
-    g_mutex_free (self->lock);
-    self->lock = NULL;
-  }
-
-  if (self->stream_finish_cond) {
-    g_cond_free (self->stream_finish_cond);
-    self->stream_finish_cond = NULL;
-  }
+  g_mutex_clear (&self->lock);
 
   G_OBJECT_CLASS (parent_class)->finalize (object);
 }
@@ -868,8 +827,7 @@ gst_stream_synchronizer_finalize (GObject * object)
 static void
 gst_stream_synchronizer_init (GstStreamSynchronizer * self)
 {
-  self->lock = g_mutex_new ();
-  self->stream_finish_cond = g_cond_new ();
+  g_mutex_init (&self->lock);
 }
 
 static void
