@@ -29,7 +29,7 @@
  * <refsect2>
  * <title>Examples</title>
  * |[
- * gst-launch -v filesrc location=test.mkv ! matroskademux name=demux ! "video/x-h264" ! queue2 ! decodebin ! subtitleoverlay name=overlay ! videoconvert ! autovideosink  demux. ! "video/x-dvd-subpicture" ! queue2 ! overlay.
+ * gst-launch -v filesrc location=test.mkv ! matroskademux name=demux ! "video/x-h264" ! queue2 ! decodebin ! subtitleoverlay name=overlay ! videoconvert ! autovideosink  demux. ! "subpicture/x-dvd" ! queue2 ! overlay.
  * ]| This will play back the given Matroska file with h264 video and subpicture subtitles.
  * </refsect2>
  */
@@ -44,8 +44,6 @@
 #include <gst/video/video.h>
 #include <string.h>
 
-#include "gst/glib-compat-private.h"
-
 GST_DEBUG_CATEGORY_STATIC (subtitle_overlay_debug);
 #define GST_CAT_DEFAULT subtitle_overlay_debug
 
@@ -101,8 +99,8 @@ static void
 do_async_done (GstSubtitleOverlay * self)
 {
   if (self->do_async) {
-    GstMessage *msg =
-        gst_message_new_async_done (GST_OBJECT_CAST (self), FALSE);
+    GstMessage *msg = gst_message_new_async_done (GST_OBJECT_CAST (self),
+        GST_CLOCK_TIME_NONE);
 
     GST_DEBUG_OBJECT (self, "Posting async-done");
     GST_BIN_CLASS (parent_class)->handle_message (GST_BIN_CAST (self), msg);
@@ -164,15 +162,8 @@ gst_subtitle_overlay_finalize (GObject * object)
 {
   GstSubtitleOverlay *self = GST_SUBTITLE_OVERLAY (object);
 
-  if (self->lock) {
-    g_mutex_free (self->lock);
-    self->lock = NULL;
-  }
-
-  if (self->factories_lock) {
-    g_mutex_free (self->factories_lock);
-    self->factories_lock = NULL;
-  }
+  g_mutex_clear (&self->lock);
+  g_mutex_clear (&self->factories_lock);
 
   if (self->factories)
     gst_plugin_feature_list_free (self->factories);
@@ -197,14 +188,17 @@ _is_renderer (GstElementFactory * factory)
 {
   const gchar *klass, *name;
 
-  klass = gst_element_factory_get_klass (factory);
+  klass =
+      gst_element_factory_get_metadata (factory, GST_ELEMENT_METADATA_KLASS);
   name = gst_plugin_feature_get_name (GST_PLUGIN_FEATURE_CAST (factory));
 
-  if (strstr (klass, "Overlay/Subtitle") != NULL ||
-      strstr (klass, "Overlay/SubPicture") != NULL)
-    return TRUE;
-  if (strcmp (name, "textoverlay") == 0)
-    return TRUE;
+  if (klass != NULL) {
+    if (strstr (klass, "Overlay/Subtitle") != NULL ||
+        strstr (klass, "Overlay/SubPicture") != NULL)
+      return TRUE;
+    if (strcmp (name, "textoverlay") == 0)
+      return TRUE;
+  }
   return FALSE;
 }
 
@@ -213,9 +207,10 @@ _is_parser (GstElementFactory * factory)
 {
   const gchar *klass;
 
-  klass = gst_element_factory_get_klass (factory);
+  klass =
+      gst_element_factory_get_metadata (factory, GST_ELEMENT_METADATA_KLASS);
 
-  if (strstr (klass, "Parser/Subtitle") != NULL)
+  if (klass != NULL && strstr (klass, "Parser/Subtitle") != NULL)
     return TRUE;
   return FALSE;
 }
@@ -362,13 +357,15 @@ _factory_filter (GstPluginFeature * feature, GstCaps ** subcaps)
 
   if (is_renderer && have_video_sink && templ_caps) {
     GST_DEBUG ("Found renderer element %s (%s) with caps %" GST_PTR_FORMAT,
-        gst_element_factory_get_longname (factory),
+        gst_element_factory_get_metadata (factory,
+            GST_ELEMENT_METADATA_LONGNAME),
         gst_plugin_feature_get_name (feature), templ_caps);
     *subcaps = gst_caps_merge (*subcaps, templ_caps);
     return TRUE;
   } else if (!is_renderer && !have_video_sink && templ_caps) {
     GST_DEBUG ("Found parser element %s (%s) with caps %" GST_PTR_FORMAT,
-        gst_element_factory_get_longname (factory),
+        gst_element_factory_get_metadata (factory,
+            GST_ELEMENT_METADATA_LONGNAME),
         gst_plugin_feature_get_name (feature), templ_caps);
     *subcaps = gst_caps_merge (*subcaps, templ_caps);
     return TRUE;
@@ -879,8 +876,9 @@ _setup_renderer (GstSubtitleOverlay * self, GstElement * renderer)
 
   if (strcmp (name, "textoverlay") == 0) {
     /* Set some textoverlay specific properties */
-    g_object_set (G_OBJECT (renderer),
-        "halign", "center", "valign", "bottom", "wait-text", FALSE, NULL);
+    gst_util_set_object_arg (G_OBJECT (renderer), "halignment", "center");
+    gst_util_set_object_arg (G_OBJECT (renderer), "valignment", "bottom");
+    g_object_set (G_OBJECT (renderer), "wait-text", FALSE, NULL);
     if (self->font_desc)
       g_object_set (G_OBJECT (renderer), "font-desc", self->font_desc, NULL);
     self->silent_property = "silent";
@@ -1182,7 +1180,7 @@ _pad_blocked_cb (GstPad * pad, GstPadProbeInfo * info, gpointer user_data)
   self->subtitle_flush = FALSE;
 
   /* Find our factories */
-  g_mutex_lock (self->factories_lock);
+  g_mutex_lock (&self->factories_lock);
   gst_subtitle_overlay_update_factory_list (self);
   if (subcaps) {
     factories =
@@ -1198,7 +1196,7 @@ _pad_blocked_cb (GstPad * pad, GstPadProbeInfo * info, gpointer user_data)
       self->subtitle_error = TRUE;
     }
   }
-  g_mutex_unlock (self->factories_lock);
+  g_mutex_unlock (&self->factories_lock);
 
   if (!subcaps) {
     _setup_passthrough (self);
@@ -1255,14 +1253,14 @@ _pad_blocked_cb (GstPad * pad, GstPadProbeInfo * info, gpointer user_data)
 
       g_assert (parser_caps != NULL);
 
-      g_mutex_lock (self->factories_lock);
+      g_mutex_lock (&self->factories_lock);
       gst_subtitle_overlay_update_factory_list (self);
       GST_DEBUG_OBJECT (self,
           "Searching overlay factories for caps %" GST_PTR_FORMAT, parser_caps);
       overlay_factories =
           gst_subtitle_overlay_get_factories_for_caps (self->factories,
           parser_caps);
-      g_mutex_unlock (self->factories_lock);
+      g_mutex_unlock (&self->factories_lock);
 
       if (!overlay_factories) {
         GST_WARNING_OBJECT (self,
@@ -1428,12 +1426,12 @@ gst_subtitle_overlay_change_state (GstElement * element,
   switch (transition) {
     case GST_STATE_CHANGE_NULL_TO_READY:
       GST_DEBUG_OBJECT (self, "State change NULL->READY");
-      g_mutex_lock (self->factories_lock);
+      g_mutex_lock (&self->factories_lock);
       if (G_UNLIKELY (!gst_subtitle_overlay_update_factory_list (self))) {
-        g_mutex_unlock (self->factories_lock);
+        g_mutex_unlock (&self->factories_lock);
         return GST_STATE_CHANGE_FAILURE;
       }
-      g_mutex_unlock (self->factories_lock);
+      g_mutex_unlock (&self->factories_lock);
 
       GST_SUBTITLE_OVERLAY_LOCK (self);
       /* Set the internal pads to blocking */
@@ -1771,7 +1769,7 @@ gst_subtitle_overlay_src_proxy_event (GstPad * proxypad, GstObject * parent,
     event = NULL;
     ret = TRUE;
   } else {
-    ret = gst_proxy_pad_event_default (proxypad, parent, event);
+    ret = gst_pad_event_default (proxypad, parent, event);
     event = NULL;
   }
 
@@ -1858,7 +1856,7 @@ gst_subtitle_overlay_video_sink_event (GstPad * pad, GstObject * parent,
       break;
   }
 
-  ret = gst_proxy_pad_event_default (pad, parent, gst_event_ref (event));
+  ret = gst_pad_event_default (pad, parent, gst_event_ref (event));
 
   if (GST_EVENT_TYPE (event) == GST_EVENT_SEGMENT) {
     GST_DEBUG_OBJECT (pad, "segment event: %" GST_PTR_FORMAT, event);
@@ -1941,7 +1939,7 @@ gst_subtitle_overlay_subtitle_sink_getcaps (GstPad * pad, GstCaps * filter)
   GstSubtitleOverlay *self = GST_SUBTITLE_OVERLAY (gst_pad_get_parent (pad));
   GstCaps *ret;
 
-  g_mutex_lock (self->factories_lock);
+  g_mutex_lock (&self->factories_lock);
   if (G_UNLIKELY (!gst_subtitle_overlay_update_factory_list (self)))
     ret = gst_caps_new_empty ();
   else if (filter)
@@ -1950,7 +1948,7 @@ gst_subtitle_overlay_subtitle_sink_getcaps (GstPad * pad, GstCaps * filter)
         GST_CAPS_INTERSECT_FIRST);
   else
     ret = gst_caps_ref (self->factory_caps);
-  g_mutex_unlock (self->factories_lock);
+  g_mutex_unlock (&self->factories_lock);
 
   GST_DEBUG_OBJECT (pad, "Returning subtitle caps %" GST_PTR_FORMAT, ret);
 
@@ -1995,10 +1993,10 @@ out:
 }
 
 static GstPadLinkReturn
-gst_subtitle_overlay_subtitle_sink_link (GstPad * pad, GstPad * peer)
+gst_subtitle_overlay_subtitle_sink_link (GstPad * pad, GstObject * parent,
+    GstPad * peer)
 {
-  GstSubtitleOverlay *self = GST_SUBTITLE_OVERLAY (gst_pad_get_parent (pad));
-  GstPadLinkReturn ret;
+  GstSubtitleOverlay *self = GST_SUBTITLE_OVERLAY (parent);
   GstCaps *caps;
 
   GST_DEBUG_OBJECT (pad, "Linking pad to peer %" GST_PTR_FORMAT, peer);
@@ -2025,17 +2023,13 @@ gst_subtitle_overlay_subtitle_sink_link (GstPad * pad, GstPad * peer)
     gst_caps_unref (caps);
   }
 
-  ret = gst_ghost_pad_link_default (pad, peer);
-
-  gst_object_unref (self);
-  return ret;
+  return GST_PAD_LINK_OK;
 }
 
 static void
-gst_subtitle_overlay_subtitle_sink_unlink (GstPad * pad)
+gst_subtitle_overlay_subtitle_sink_unlink (GstPad * pad, GstObject * parent)
 {
-  GstSubtitleOverlay *self =
-      GST_SUBTITLE_OVERLAY (gst_object_ref (GST_PAD_PARENT (pad)));
+  GstSubtitleOverlay *self = GST_SUBTITLE_OVERLAY (parent);
 
   /* FIXME: Can't use gst_pad_get_parent() here because this is called with
    * the object lock from state changes
@@ -2044,16 +2038,12 @@ gst_subtitle_overlay_subtitle_sink_unlink (GstPad * pad)
   GST_DEBUG_OBJECT (pad, "Pad unlinking");
   gst_caps_replace (&self->subcaps, NULL);
 
-  gst_ghost_pad_unlink_default (pad);
-
   GST_SUBTITLE_OVERLAY_LOCK (self);
   self->subtitle_error = FALSE;
 
   block_subtitle (self);
   block_video (self);
   GST_SUBTITLE_OVERLAY_UNLOCK (self);
-
-  gst_object_unref (self);
 }
 
 static gboolean
@@ -2116,7 +2106,7 @@ gst_subtitle_overlay_subtitle_sink_event (GstPad * pad, GstObject * parent,
       break;
   }
 
-  ret = gst_proxy_pad_event_default (pad, parent, gst_event_ref (event));
+  ret = gst_pad_event_default (pad, parent, gst_event_ref (event));
 
   if (GST_EVENT_TYPE (event) == GST_EVENT_SEGMENT) {
     const GstSegment *eventsegment;
@@ -2234,8 +2224,8 @@ gst_subtitle_overlay_init (GstSubtitleOverlay * self)
   GstPadTemplate *templ;
   GstPad *proxypad = NULL;
 
-  self->lock = g_mutex_new ();
-  self->factories_lock = g_mutex_new ();
+  g_mutex_init (&self->lock);
+  g_mutex_init (&self->factories_lock);
 
   templ = gst_static_pad_template_get (&srctemplate);
   self->srcpad = gst_ghost_pad_new_no_target_from_template ("src", templ);
