@@ -33,13 +33,16 @@
 
 static void videoconvert_convert_generic (VideoConvert * convert,
     GstVideoFrame * dest, const GstVideoFrame * src);
-static void videoconvert_convert_matrix (VideoConvert * convert);
-static void videoconvert_convert_matrix16 (VideoConvert * convert);
+static void videoconvert_convert_matrix (VideoConvert * convert,
+    guint8 * pixels);
+static void videoconvert_convert_matrix16 (VideoConvert * convert,
+    guint16 * pixels);
 static gboolean videoconvert_convert_lookup_fastpath (VideoConvert * convert);
 static gboolean videoconvert_convert_compute_matrix (VideoConvert * convert);
-static void videoconvert_dither_none (VideoConvert * convert, int j);
-static void videoconvert_dither_verterr (VideoConvert * convert, int j);
-static void videoconvert_dither_halftone (VideoConvert * convert, int j);
+static void videoconvert_dither_verterr (VideoConvert * convert,
+    guint16 * pixels, int j);
+static void videoconvert_dither_halftone (VideoConvert * convert,
+    guint16 * pixels, int j);
 
 
 VideoConvert *
@@ -52,7 +55,7 @@ videoconvert_convert_new (GstVideoInfo * in_info, GstVideoInfo * out_info)
 
   convert->in_info = *in_info;
   convert->out_info = *out_info;
-  convert->dither16 = videoconvert_dither_none;
+  convert->dither16 = NULL;
 
   if (!videoconvert_convert_lookup_fastpath (convert)) {
     convert->convert = videoconvert_convert_generic;
@@ -117,7 +120,7 @@ videoconvert_convert_set_dither (VideoConvert * convert, int type)
   switch (type) {
     case 0:
     default:
-      convert->dither16 = videoconvert_dither_none;
+      convert->dither16 = NULL;
       break;
     case 1:
       convert->dither16 = videoconvert_dither_verterr;
@@ -135,18 +138,17 @@ videoconvert_convert_convert (VideoConvert * convert,
   convert->convert (convert, dest, src);
 }
 
-void
-videoconvert_convert_matrix (VideoConvert * convert)
+static void
+videoconvert_convert_matrix (VideoConvert * convert, guint8 * pixels)
 {
   int i;
   int r, g, b;
   int y, u, v;
-  guint8 *tmpline = convert->tmpline;
 
   for (i = 0; i < convert->width; i++) {
-    r = tmpline[i * 4 + 1];
-    g = tmpline[i * 4 + 2];
-    b = tmpline[i * 4 + 3];
+    r = pixels[i * 4 + 1];
+    g = pixels[i * 4 + 2];
+    b = pixels[i * 4 + 3];
 
     y = (convert->cmatrix[0][0] * r + convert->cmatrix[0][1] * g +
         convert->cmatrix[0][2] * b + convert->cmatrix[0][3]) >> 8;
@@ -155,24 +157,23 @@ videoconvert_convert_matrix (VideoConvert * convert)
     v = (convert->cmatrix[2][0] * r + convert->cmatrix[2][1] * g +
         convert->cmatrix[2][2] * b + convert->cmatrix[2][3]) >> 8;
 
-    tmpline[i * 4 + 1] = CLAMP (y, 0, 255);
-    tmpline[i * 4 + 2] = CLAMP (u, 0, 255);
-    tmpline[i * 4 + 3] = CLAMP (v, 0, 255);
+    pixels[i * 4 + 1] = CLAMP (y, 0, 255);
+    pixels[i * 4 + 2] = CLAMP (u, 0, 255);
+    pixels[i * 4 + 3] = CLAMP (v, 0, 255);
   }
 }
 
-void
-videoconvert_convert_matrix16 (VideoConvert * convert)
+static void
+videoconvert_convert_matrix16 (VideoConvert * convert, guint16 * pixels)
 {
   int i;
   int r, g, b;
   int y, u, v;
-  guint16 *tmpline = convert->tmpline16;
 
   for (i = 0; i < convert->width; i++) {
-    r = tmpline[i * 4 + 1];
-    g = tmpline[i * 4 + 2];
-    b = tmpline[i * 4 + 3];
+    r = pixels[i * 4 + 1];
+    g = pixels[i * 4 + 2];
+    b = pixels[i * 4 + 3];
 
     y = (convert->cmatrix[0][0] * r + convert->cmatrix[0][1] * g +
         convert->cmatrix[0][2] * b + convert->cmatrix[0][3]) >> 8;
@@ -181,66 +182,9 @@ videoconvert_convert_matrix16 (VideoConvert * convert)
     v = (convert->cmatrix[2][0] * r + convert->cmatrix[2][1] * g +
         convert->cmatrix[2][2] * b + convert->cmatrix[2][3]) >> 8;
 
-    tmpline[i * 4 + 1] = CLAMP (y, 0, 65535);
-    tmpline[i * 4 + 2] = CLAMP (u, 0, 65535);
-    tmpline[i * 4 + 3] = CLAMP (v, 0, 65535);
-  }
-}
-
-static void
-matrix_identity (VideoConvert * convert)
-{
-  /* do nothing */
-}
-
-static void
-get_offset_scale (const GstVideoFormatInfo * finfo, GstVideoColorRange range,
-    gint depth, gint offset[4], gint scale[4])
-{
-  gint minL, minC, baseL, baseC, maxL, maxC;
-
-  if (depth < 8)
-    depth = 8;
-
-  switch (range) {
-    default:
-    case GST_VIDEO_COLOR_RANGE_0_255:
-      minL = minC = 0;
-      baseL = 0;
-      baseC = 128 << (depth - 8);
-      maxL = (1 << depth) - 1;
-      maxC = (1 << depth) - 1;
-      break;
-    case GST_VIDEO_COLOR_RANGE_16_235:
-      minL = 16 << (depth - 8);
-      minC = 16 << (depth - 8);
-      baseL = minL;
-      baseC = 128 << (depth - 8);
-      maxL = 235 << (depth - 8);
-      maxC = 240 << (depth - 8);
-      break;
-  }
-
-  if (GST_VIDEO_FORMAT_INFO_IS_YUV (finfo)) {
-    offset[0] = 0;
-    offset[1] = baseL;
-    offset[2] = offset[3] = baseC;
-    scale[0] = 0;
-    scale[1] = maxL - minL;
-    scale[2] = scale[3] = maxC - minC;
-  } else if (GST_VIDEO_FORMAT_INFO_IS_RGB (finfo)) {
-    offset[0] = 0;
-    offset[1] = offset[2] = offset[3] = baseL;
-    scale[0] = 0;
-    scale[1] = scale[2] = scale[3] = maxL - minL;
-  } else if (GST_VIDEO_FORMAT_INFO_IS_GRAY (finfo)) {
-    offset[0] = offset[2] = offset[3] = 0;
-    offset[1] = baseL;
-    scale[0] = scale[2] = scale[3] = 0;
-    scale[1] = maxL - minL;
-  } else {
-    offset[0] = offset[1] = offset[2] = offset[3] = 0;
-    scale[0] = scale[1] = scale[2] = scale[3] = (maxL - minL);
+    pixels[i * 4 + 1] = CLAMP (y, 0, 65535);
+    pixels[i * 4 + 2] = CLAMP (u, 0, 65535);
+    pixels[i * 4 + 3] = CLAMP (v, 0, 65535);
   }
 }
 
@@ -273,6 +217,7 @@ get_Kr_Kb (GstVideoColorMatrix matrix, gdouble * Kr, gdouble * Kb)
       *Kb = 0.087;
       break;
   }
+  GST_DEBUG ("matrix: %d, Kr %f, Kb %f", matrix, *Kr, *Kb);
   return res;
 }
 
@@ -283,9 +228,9 @@ videoconvert_convert_compute_matrix (VideoConvert * convert)
   ColorMatrix dst;
   gint i, j;
   const GstVideoFormatInfo *sfinfo, *dfinfo;
-  gint depth;
+  const GstVideoFormatInfo *suinfo, *duinfo;
   gint offset[4], scale[4];
-  gdouble Kr, Kb;
+  gdouble Kr = 0, Kb = 0;
 
   in_info = &convert->in_info;
   out_info = &convert->out_info;
@@ -299,34 +244,45 @@ videoconvert_convert_compute_matrix (VideoConvert * convert)
   if (dfinfo->pack_func == NULL)
     goto no_pack_func;
 
-  convert->in_bits =
-      GST_VIDEO_FORMAT_INFO_DEPTH (gst_video_format_get_info
-      (sfinfo->unpack_format), 0);
-  convert->out_bits =
-      GST_VIDEO_FORMAT_INFO_DEPTH (gst_video_format_get_info
-      (dfinfo->unpack_format), 0);
+  suinfo = gst_video_format_get_info (sfinfo->unpack_format);
+  duinfo = gst_video_format_get_info (dfinfo->unpack_format);
+
+  convert->in_bits = GST_VIDEO_FORMAT_INFO_DEPTH (suinfo, 0);
+  convert->out_bits = GST_VIDEO_FORMAT_INFO_DEPTH (duinfo, 0);
+
+  GST_DEBUG ("in bits %d, out bits %d", convert->in_bits, convert->out_bits);
 
   if (in_info->colorimetry.range == out_info->colorimetry.range &&
       in_info->colorimetry.matrix == out_info->colorimetry.matrix) {
     GST_DEBUG ("using identity color transform");
-    convert->matrix = matrix_identity;
-    convert->matrix16 = matrix_identity;
+    convert->matrix = NULL;
+    convert->matrix16 = NULL;
     return TRUE;
   }
 
-  if (convert->in_bits == 16 || convert->out_bits == 16)
-    depth = 16;
-  else
-    depth = 8;
+  /* calculate intermediate format for the matrix. When unpacking, we expand
+   * input to 16 when one of the inputs is 16 bits */
+  if (convert->in_bits == 16 || convert->out_bits == 16) {
+    if (GST_VIDEO_FORMAT_INFO_IS_RGB (suinfo))
+      suinfo = gst_video_format_get_info (GST_VIDEO_FORMAT_ARGB64);
+    else
+      suinfo = gst_video_format_get_info (GST_VIDEO_FORMAT_AYUV64);
+
+    if (GST_VIDEO_FORMAT_INFO_IS_RGB (duinfo))
+      duinfo = gst_video_format_get_info (GST_VIDEO_FORMAT_ARGB64);
+    else
+      duinfo = gst_video_format_get_info (GST_VIDEO_FORMAT_AYUV64);
+  }
 
   color_matrix_set_identity (&dst);
 
   /* 1, bring color components to [0..1.0] range */
-  get_offset_scale (sfinfo, in_info->colorimetry.range, depth, offset, scale);
+  gst_video_color_range_offsets (in_info->colorimetry.range, suinfo, offset,
+      scale);
+  color_matrix_offset_components (&dst, -offset[0], -offset[1], -offset[2]);
 
-  color_matrix_offset_components (&dst, -offset[1], -offset[2], -offset[3]);
-  color_matrix_scale_components (&dst, 1 / ((float) scale[1]),
-      1 / ((float) scale[2]), 1 / ((float) scale[3]));
+  color_matrix_scale_components (&dst, 1 / ((float) scale[0]),
+      1 / ((float) scale[1]), 1 / ((float) scale[2]));
 
   /* 2. bring components to R'G'B' space */
   if (get_Kr_Kb (in_info->colorimetry.matrix, &Kr, &Kb))
@@ -345,10 +301,12 @@ videoconvert_convert_compute_matrix (VideoConvert * convert)
     color_matrix_RGB_to_YCbCr (&dst, Kr, Kb);
 
   /* 8, bring color components to nominal range */
-  get_offset_scale (dfinfo, out_info->colorimetry.range, depth, offset, scale);
-  color_matrix_scale_components (&dst, (float) scale[1], (float) scale[2],
-      (float) scale[3]);
-  color_matrix_offset_components (&dst, offset[1], offset[2], offset[3]);
+  gst_video_color_range_offsets (out_info->colorimetry.range, duinfo, offset,
+      scale);
+  color_matrix_scale_components (&dst, (float) scale[0], (float) scale[1],
+      (float) scale[2]);
+
+  color_matrix_offset_components (&dst, offset[0], offset[1], offset[2]);
 
   /* because we're doing 8-bit matrix coefficients */
   color_matrix_scale_components (&dst, 256.0, 256.0, 256.0);
@@ -387,12 +345,7 @@ no_pack_func:
 }
 
 static void
-videoconvert_dither_none (VideoConvert * convert, int j)
-{
-}
-
-static void
-videoconvert_dither_verterr (VideoConvert * convert, int j)
+videoconvert_dither_verterr (VideoConvert * convert, guint16 * pixels, int j)
 {
   int i;
   guint16 *tmpline = convert->tmpline16;
@@ -409,7 +362,7 @@ videoconvert_dither_verterr (VideoConvert * convert, int j)
 }
 
 static void
-videoconvert_dither_halftone (VideoConvert * convert, int j)
+videoconvert_dither_halftone (VideoConvert * convert, guint16 * pixels, int j)
 {
   int i;
   guint16 *tmpline = convert->tmpline16;
@@ -468,10 +421,13 @@ videoconvert_convert_generic (VideoConvert * convert, GstVideoFrame * dest,
     }
 
     if (out_bits == 16 || in_bits == 16) {
-      convert->matrix16 (convert);
-      convert->dither16 (convert, j);
+      if (convert->matrix16)
+        convert->matrix16 (convert, convert->tmpline16);
+      if (convert->dither16)
+        convert->dither16 (convert, convert->tmpline16, j);
     } else {
-      convert->matrix (convert);
+      if (convert->matrix)
+        convert->matrix (convert, convert->tmpline);
     }
 
     if (out_bits == 16) {
@@ -522,7 +478,7 @@ convert_I420_YUY2 (VideoConvert * convert, GstVideoFrame * dest,
   gint height = convert->height;
 
   for (i = 0; i < GST_ROUND_DOWN_2 (height); i += 2) {
-    cogorc_convert_I420_YUY2 (FRAME_GET_LINE (dest, i),
+    video_convert_orc_convert_I420_YUY2 (FRAME_GET_LINE (dest, i),
         FRAME_GET_LINE (dest, i + 1),
         FRAME_GET_Y_LINE (src, i),
         FRAME_GET_Y_LINE (src, i + 1),
@@ -546,7 +502,7 @@ convert_I420_UYVY (VideoConvert * convert, GstVideoFrame * dest,
   gint height = convert->height;
 
   for (i = 0; i < GST_ROUND_DOWN_2 (height); i += 2) {
-    cogorc_convert_I420_UYVY (FRAME_GET_LINE (dest, i),
+    video_convert_orc_convert_I420_UYVY (FRAME_GET_LINE (dest, i),
         FRAME_GET_LINE (dest, i + 1),
         FRAME_GET_Y_LINE (src, i),
         FRAME_GET_Y_LINE (src, i + 1),
@@ -570,7 +526,7 @@ convert_I420_AYUV (VideoConvert * convert, GstVideoFrame * dest,
   gint height = convert->height;
 
   for (i = 0; i < GST_ROUND_DOWN_2 (height); i += 2) {
-    cogorc_convert_I420_AYUV (FRAME_GET_LINE (dest, i),
+    video_convert_orc_convert_I420_AYUV (FRAME_GET_LINE (dest, i),
         FRAME_GET_LINE (dest, i + 1),
         FRAME_GET_Y_LINE (src, i),
         FRAME_GET_Y_LINE (src, i + 1),
@@ -591,15 +547,16 @@ convert_I420_Y42B (VideoConvert * convert, GstVideoFrame * dest,
   gint width = convert->width;
   gint height = convert->height;
 
-  cogorc_memcpy_2d (FRAME_GET_Y_LINE (dest, 0), FRAME_GET_Y_STRIDE (dest),
-      FRAME_GET_Y_LINE (src, 0), FRAME_GET_Y_STRIDE (src), width, height);
+  video_convert_orc_memcpy_2d (FRAME_GET_Y_LINE (dest, 0),
+      FRAME_GET_Y_STRIDE (dest), FRAME_GET_Y_LINE (src, 0),
+      FRAME_GET_Y_STRIDE (src), width, height);
 
-  cogorc_planar_chroma_420_422 (FRAME_GET_U_LINE (dest, 0),
+  video_convert_orc_planar_chroma_420_422 (FRAME_GET_U_LINE (dest, 0),
       2 * FRAME_GET_U_STRIDE (dest), FRAME_GET_U_LINE (dest, 1),
       2 * FRAME_GET_U_STRIDE (dest), FRAME_GET_U_LINE (src, 0),
       FRAME_GET_U_STRIDE (src), (width + 1) / 2, height / 2);
 
-  cogorc_planar_chroma_420_422 (FRAME_GET_V_LINE (dest, 0),
+  video_convert_orc_planar_chroma_420_422 (FRAME_GET_V_LINE (dest, 0),
       2 * FRAME_GET_V_STRIDE (dest), FRAME_GET_V_LINE (dest, 1),
       2 * FRAME_GET_V_STRIDE (dest), FRAME_GET_V_LINE (src, 0),
       FRAME_GET_V_STRIDE (src), (width + 1) / 2, height / 2);
@@ -612,15 +569,16 @@ convert_I420_Y444 (VideoConvert * convert, GstVideoFrame * dest,
   gint width = convert->width;
   gint height = convert->height;
 
-  cogorc_memcpy_2d (FRAME_GET_Y_LINE (dest, 0), FRAME_GET_Y_STRIDE (dest),
-      FRAME_GET_Y_LINE (src, 0), FRAME_GET_Y_STRIDE (src), width, height);
+  video_convert_orc_memcpy_2d (FRAME_GET_Y_LINE (dest, 0),
+      FRAME_GET_Y_STRIDE (dest), FRAME_GET_Y_LINE (src, 0),
+      FRAME_GET_Y_STRIDE (src), width, height);
 
-  cogorc_planar_chroma_420_444 (FRAME_GET_U_LINE (dest, 0),
+  video_convert_orc_planar_chroma_420_444 (FRAME_GET_U_LINE (dest, 0),
       2 * FRAME_GET_U_STRIDE (dest), FRAME_GET_U_LINE (dest, 1),
       2 * FRAME_GET_U_STRIDE (dest), FRAME_GET_U_LINE (src, 0),
       FRAME_GET_U_STRIDE (src), (width + 1) / 2, height / 2);
 
-  cogorc_planar_chroma_420_444 (FRAME_GET_V_LINE (dest, 0),
+  video_convert_orc_planar_chroma_420_444 (FRAME_GET_V_LINE (dest, 0),
       2 * FRAME_GET_V_STRIDE (dest), FRAME_GET_V_LINE (dest, 1),
       2 * FRAME_GET_V_STRIDE (dest), FRAME_GET_V_LINE (src, 0),
       FRAME_GET_V_STRIDE (src), (width + 1) / 2, height / 2);
@@ -645,7 +603,7 @@ convert_YUY2_I420 (VideoConvert * convert, GstVideoFrame * dest,
     h--;
 
   for (i = 0; i < h; i += 2) {
-    cogorc_convert_YUY2_I420 (FRAME_GET_Y_LINE (dest, i),
+    video_convert_orc_convert_YUY2_I420 (FRAME_GET_Y_LINE (dest, i),
         FRAME_GET_Y_LINE (dest, i + 1),
         FRAME_GET_U_LINE (dest, i >> 1),
         FRAME_GET_V_LINE (dest, i >> 1),
@@ -666,7 +624,7 @@ convert_YUY2_AYUV (VideoConvert * convert, GstVideoFrame * dest,
   gint width = convert->width;
   gint height = convert->height;
 
-  cogorc_convert_YUY2_AYUV (FRAME_GET_LINE (dest, 0),
+  video_convert_orc_convert_YUY2_AYUV (FRAME_GET_LINE (dest, 0),
       FRAME_GET_STRIDE (dest), FRAME_GET_LINE (src, 0),
       FRAME_GET_STRIDE (src), (width + 1) / 2,
       height & 1 ? height - 1 : height);
@@ -685,7 +643,7 @@ convert_YUY2_Y42B (VideoConvert * convert, GstVideoFrame * dest,
   gint width = convert->width;
   gint height = convert->height;
 
-  cogorc_convert_YUY2_Y42B (FRAME_GET_Y_LINE (dest, 0),
+  video_convert_orc_convert_YUY2_Y42B (FRAME_GET_Y_LINE (dest, 0),
       FRAME_GET_Y_STRIDE (dest), FRAME_GET_U_LINE (dest, 0),
       FRAME_GET_U_STRIDE (dest), FRAME_GET_V_LINE (dest, 0),
       FRAME_GET_V_STRIDE (dest), FRAME_GET_LINE (src, 0),
@@ -699,7 +657,7 @@ convert_YUY2_Y444 (VideoConvert * convert, GstVideoFrame * dest,
   gint width = convert->width;
   gint height = convert->height;
 
-  cogorc_convert_YUY2_Y444 (FRAME_GET_COMP_LINE (dest, 0, 0),
+  video_convert_orc_convert_YUY2_Y444 (FRAME_GET_COMP_LINE (dest, 0, 0),
       FRAME_GET_COMP_STRIDE (dest, 0), FRAME_GET_COMP_LINE (dest, 1, 0),
       FRAME_GET_COMP_STRIDE (dest, 1), FRAME_GET_COMP_LINE (dest, 2, 0),
       FRAME_GET_COMP_STRIDE (dest, 2), FRAME_GET_LINE (src, 0),
@@ -716,7 +674,7 @@ convert_UYVY_I420 (VideoConvert * convert, GstVideoFrame * dest,
   gint height = convert->height;
 
   for (i = 0; i < GST_ROUND_DOWN_2 (height); i += 2) {
-    cogorc_convert_UYVY_I420 (FRAME_GET_COMP_LINE (dest, 0, i),
+    video_convert_orc_convert_UYVY_I420 (FRAME_GET_COMP_LINE (dest, 0, i),
         FRAME_GET_COMP_LINE (dest, 0, i + 1),
         FRAME_GET_COMP_LINE (dest, 1, i >> 1),
         FRAME_GET_COMP_LINE (dest, 2, i >> 1),
@@ -737,7 +695,7 @@ convert_UYVY_AYUV (VideoConvert * convert, GstVideoFrame * dest,
   gint width = convert->width;
   gint height = convert->height;
 
-  cogorc_convert_UYVY_AYUV (FRAME_GET_LINE (dest, 0),
+  video_convert_orc_convert_UYVY_AYUV (FRAME_GET_LINE (dest, 0),
       FRAME_GET_STRIDE (dest), FRAME_GET_LINE (src, 0),
       FRAME_GET_STRIDE (src), (width + 1) / 2,
       height & 1 ? height - 1 : height);
@@ -756,7 +714,7 @@ convert_UYVY_YUY2 (VideoConvert * convert, GstVideoFrame * dest,
   gint width = convert->width;
   gint height = convert->height;
 
-  cogorc_convert_UYVY_YUY2 (FRAME_GET_LINE (dest, 0),
+  video_convert_orc_convert_UYVY_YUY2 (FRAME_GET_LINE (dest, 0),
       FRAME_GET_STRIDE (dest), FRAME_GET_LINE (src, 0),
       FRAME_GET_STRIDE (src), (width + 1) / 2, height);
 }
@@ -768,7 +726,7 @@ convert_UYVY_Y42B (VideoConvert * convert, GstVideoFrame * dest,
   gint width = convert->width;
   gint height = convert->height;
 
-  cogorc_convert_UYVY_Y42B (FRAME_GET_Y_LINE (dest, 0),
+  video_convert_orc_convert_UYVY_Y42B (FRAME_GET_Y_LINE (dest, 0),
       FRAME_GET_Y_STRIDE (dest), FRAME_GET_U_LINE (dest, 0),
       FRAME_GET_U_STRIDE (dest), FRAME_GET_V_LINE (dest, 0),
       FRAME_GET_V_STRIDE (dest), FRAME_GET_LINE (src, 0),
@@ -782,7 +740,7 @@ convert_UYVY_Y444 (VideoConvert * convert, GstVideoFrame * dest,
   gint width = convert->width;
   gint height = convert->height;
 
-  cogorc_convert_UYVY_Y444 (FRAME_GET_Y_LINE (dest, 0),
+  video_convert_orc_convert_UYVY_Y444 (FRAME_GET_Y_LINE (dest, 0),
       FRAME_GET_Y_STRIDE (dest), FRAME_GET_U_LINE (dest, 0),
       FRAME_GET_U_STRIDE (dest), FRAME_GET_V_LINE (dest, 0),
       FRAME_GET_V_STRIDE (dest), FRAME_GET_LINE (src, 0),
@@ -796,7 +754,7 @@ convert_AYUV_I420 (VideoConvert * convert, GstVideoFrame * dest,
   gint width = convert->width;
   gint height = convert->height;
 
-  cogorc_convert_AYUV_I420 (FRAME_GET_Y_LINE (dest, 0),
+  video_convert_orc_convert_AYUV_I420 (FRAME_GET_Y_LINE (dest, 0),
       2 * FRAME_GET_Y_STRIDE (dest), FRAME_GET_Y_LINE (dest, 1),
       2 * FRAME_GET_Y_STRIDE (dest), FRAME_GET_U_LINE (dest, 0),
       FRAME_GET_U_STRIDE (dest), FRAME_GET_V_LINE (dest, 0),
@@ -812,7 +770,7 @@ convert_AYUV_YUY2 (VideoConvert * convert, GstVideoFrame * dest,
   gint width = convert->width;
   gint height = convert->height;
 
-  cogorc_convert_AYUV_YUY2 (FRAME_GET_LINE (dest, 0),
+  video_convert_orc_convert_AYUV_YUY2 (FRAME_GET_LINE (dest, 0),
       FRAME_GET_STRIDE (dest), FRAME_GET_LINE (src, 0),
       FRAME_GET_STRIDE (src), width / 2, height);
 }
@@ -824,7 +782,7 @@ convert_AYUV_UYVY (VideoConvert * convert, GstVideoFrame * dest,
   gint width = convert->width;
   gint height = convert->height;
 
-  cogorc_convert_AYUV_UYVY (FRAME_GET_LINE (dest, 0),
+  video_convert_orc_convert_AYUV_UYVY (FRAME_GET_LINE (dest, 0),
       FRAME_GET_STRIDE (dest), FRAME_GET_LINE (src, 0),
       FRAME_GET_STRIDE (src), width / 2, height);
 }
@@ -836,7 +794,7 @@ convert_AYUV_Y42B (VideoConvert * convert, GstVideoFrame * dest,
   gint width = convert->width;
   gint height = convert->height;
 
-  cogorc_convert_AYUV_Y42B (FRAME_GET_Y_LINE (dest, 0),
+  video_convert_orc_convert_AYUV_Y42B (FRAME_GET_Y_LINE (dest, 0),
       FRAME_GET_Y_STRIDE (dest), FRAME_GET_U_LINE (dest, 0),
       FRAME_GET_U_STRIDE (dest), FRAME_GET_V_LINE (dest, 0),
       FRAME_GET_V_STRIDE (dest), FRAME_GET_LINE (src, 0),
@@ -857,7 +815,7 @@ convert_AYUV_Y444 (VideoConvert * convert, GstVideoFrame * dest,
   gint width = convert->width;
   gint height = convert->height;
 
-  cogorc_convert_AYUV_Y444 (FRAME_GET_Y_LINE (dest, 0),
+  video_convert_orc_convert_AYUV_Y444 (FRAME_GET_Y_LINE (dest, 0),
       FRAME_GET_Y_STRIDE (dest), FRAME_GET_U_LINE (dest, 0),
       FRAME_GET_U_STRIDE (dest), FRAME_GET_V_LINE (dest, 0),
       FRAME_GET_V_STRIDE (dest), FRAME_GET_LINE (src, 0),
@@ -871,15 +829,16 @@ convert_Y42B_I420 (VideoConvert * convert, GstVideoFrame * dest,
   gint width = convert->width;
   gint height = convert->height;
 
-  cogorc_memcpy_2d (FRAME_GET_Y_LINE (dest, 0), FRAME_GET_Y_STRIDE (dest),
-      FRAME_GET_Y_LINE (src, 0), FRAME_GET_Y_STRIDE (src), width, height);
+  video_convert_orc_memcpy_2d (FRAME_GET_Y_LINE (dest, 0),
+      FRAME_GET_Y_STRIDE (dest), FRAME_GET_Y_LINE (src, 0),
+      FRAME_GET_Y_STRIDE (src), width, height);
 
-  cogorc_planar_chroma_422_420 (FRAME_GET_U_LINE (dest, 0),
+  video_convert_orc_planar_chroma_422_420 (FRAME_GET_U_LINE (dest, 0),
       FRAME_GET_U_STRIDE (dest), FRAME_GET_U_LINE (src, 0),
       2 * FRAME_GET_U_STRIDE (src), FRAME_GET_U_LINE (src, 1),
       2 * FRAME_GET_U_STRIDE (src), (width + 1) / 2, height / 2);
 
-  cogorc_planar_chroma_422_420 (FRAME_GET_V_LINE (dest, 0),
+  video_convert_orc_planar_chroma_422_420 (FRAME_GET_V_LINE (dest, 0),
       FRAME_GET_V_STRIDE (dest), FRAME_GET_V_LINE (src, 0),
       2 * FRAME_GET_V_STRIDE (src), FRAME_GET_V_LINE (src, 1),
       2 * FRAME_GET_V_STRIDE (src), (width + 1) / 2, height / 2);
@@ -898,14 +857,15 @@ convert_Y42B_Y444 (VideoConvert * convert, GstVideoFrame * dest,
   gint width = convert->width;
   gint height = convert->height;
 
-  cogorc_memcpy_2d (FRAME_GET_Y_LINE (dest, 0), FRAME_GET_Y_STRIDE (dest),
-      FRAME_GET_Y_LINE (src, 0), FRAME_GET_Y_STRIDE (src), width, height);
+  video_convert_orc_memcpy_2d (FRAME_GET_Y_LINE (dest, 0),
+      FRAME_GET_Y_STRIDE (dest), FRAME_GET_Y_LINE (src, 0),
+      FRAME_GET_Y_STRIDE (src), width, height);
 
-  cogorc_planar_chroma_422_444 (FRAME_GET_U_LINE (dest, 0),
+  video_convert_orc_planar_chroma_422_444 (FRAME_GET_U_LINE (dest, 0),
       FRAME_GET_U_STRIDE (dest), FRAME_GET_U_LINE (src, 0),
       FRAME_GET_U_STRIDE (src), (width + 1) / 2, height);
 
-  cogorc_planar_chroma_422_444 (FRAME_GET_V_LINE (dest, 0),
+  video_convert_orc_planar_chroma_422_444 (FRAME_GET_V_LINE (dest, 0),
       FRAME_GET_V_STRIDE (dest), FRAME_GET_V_LINE (src, 0),
       FRAME_GET_V_STRIDE (src), (width + 1) / 2, height);
 }
@@ -917,7 +877,7 @@ convert_Y42B_YUY2 (VideoConvert * convert, GstVideoFrame * dest,
   gint width = convert->width;
   gint height = convert->height;
 
-  cogorc_convert_Y42B_YUY2 (FRAME_GET_LINE (dest, 0),
+  video_convert_orc_convert_Y42B_YUY2 (FRAME_GET_LINE (dest, 0),
       FRAME_GET_STRIDE (dest), FRAME_GET_Y_LINE (src, 0),
       FRAME_GET_Y_STRIDE (src), FRAME_GET_U_LINE (src, 0),
       FRAME_GET_U_STRIDE (src), FRAME_GET_V_LINE (src, 0),
@@ -931,7 +891,7 @@ convert_Y42B_UYVY (VideoConvert * convert, GstVideoFrame * dest,
   gint width = convert->width;
   gint height = convert->height;
 
-  cogorc_convert_Y42B_UYVY (FRAME_GET_LINE (dest, 0),
+  video_convert_orc_convert_Y42B_UYVY (FRAME_GET_LINE (dest, 0),
       FRAME_GET_STRIDE (dest), FRAME_GET_Y_LINE (src, 0),
       FRAME_GET_Y_STRIDE (src), FRAME_GET_U_LINE (src, 0),
       FRAME_GET_U_STRIDE (src), FRAME_GET_V_LINE (src, 0),
@@ -945,7 +905,7 @@ convert_Y42B_AYUV (VideoConvert * convert, GstVideoFrame * dest,
   gint width = convert->width;
   gint height = convert->height;
 
-  cogorc_convert_Y42B_AYUV (FRAME_GET_LINE (dest, 0),
+  video_convert_orc_convert_Y42B_AYUV (FRAME_GET_LINE (dest, 0),
       FRAME_GET_STRIDE (dest), FRAME_GET_Y_LINE (src, 0),
       FRAME_GET_Y_STRIDE (src), FRAME_GET_U_LINE (src, 0),
       FRAME_GET_U_STRIDE (src), FRAME_GET_V_LINE (src, 0),
@@ -959,15 +919,16 @@ convert_Y444_I420 (VideoConvert * convert, GstVideoFrame * dest,
   gint width = convert->width;
   gint height = convert->height;
 
-  cogorc_memcpy_2d (FRAME_GET_Y_LINE (dest, 0), FRAME_GET_Y_STRIDE (dest),
-      FRAME_GET_Y_LINE (src, 0), FRAME_GET_Y_STRIDE (src), width, height);
+  video_convert_orc_memcpy_2d (FRAME_GET_Y_LINE (dest, 0),
+      FRAME_GET_Y_STRIDE (dest), FRAME_GET_Y_LINE (src, 0),
+      FRAME_GET_Y_STRIDE (src), width, height);
 
-  cogorc_planar_chroma_444_420 (FRAME_GET_U_LINE (dest, 0),
+  video_convert_orc_planar_chroma_444_420 (FRAME_GET_U_LINE (dest, 0),
       FRAME_GET_U_STRIDE (dest), FRAME_GET_U_LINE (src, 0),
       2 * FRAME_GET_U_STRIDE (src), FRAME_GET_U_LINE (src, 1),
       2 * FRAME_GET_U_STRIDE (src), (width + 1) / 2, height / 2);
 
-  cogorc_planar_chroma_444_420 (FRAME_GET_V_LINE (dest, 0),
+  video_convert_orc_planar_chroma_444_420 (FRAME_GET_V_LINE (dest, 0),
       FRAME_GET_V_STRIDE (dest), FRAME_GET_V_LINE (src, 0),
       2 * FRAME_GET_V_STRIDE (src), FRAME_GET_V_LINE (src, 1),
       2 * FRAME_GET_V_STRIDE (src), (width + 1) / 2, height / 2);
@@ -986,14 +947,15 @@ convert_Y444_Y42B (VideoConvert * convert, GstVideoFrame * dest,
   gint width = convert->width;
   gint height = convert->height;
 
-  cogorc_memcpy_2d (FRAME_GET_Y_LINE (dest, 0), FRAME_GET_Y_STRIDE (dest),
-      FRAME_GET_Y_LINE (src, 0), FRAME_GET_Y_STRIDE (src), width, height);
+  video_convert_orc_memcpy_2d (FRAME_GET_Y_LINE (dest, 0),
+      FRAME_GET_Y_STRIDE (dest), FRAME_GET_Y_LINE (src, 0),
+      FRAME_GET_Y_STRIDE (src), width, height);
 
-  cogorc_planar_chroma_444_422 (FRAME_GET_U_LINE (dest, 0),
+  video_convert_orc_planar_chroma_444_422 (FRAME_GET_U_LINE (dest, 0),
       FRAME_GET_U_STRIDE (dest), FRAME_GET_U_LINE (src, 0),
       FRAME_GET_U_STRIDE (src), (width + 1) / 2, height);
 
-  cogorc_planar_chroma_444_422 (FRAME_GET_V_LINE (dest, 0),
+  video_convert_orc_planar_chroma_444_422 (FRAME_GET_V_LINE (dest, 0),
       FRAME_GET_V_STRIDE (dest), FRAME_GET_V_LINE (src, 0),
       FRAME_GET_V_STRIDE (src), (width + 1) / 2, height);
 }
@@ -1005,7 +967,7 @@ convert_Y444_YUY2 (VideoConvert * convert, GstVideoFrame * dest,
   gint width = convert->width;
   gint height = convert->height;
 
-  cogorc_convert_Y444_YUY2 (FRAME_GET_LINE (dest, 0),
+  video_convert_orc_convert_Y444_YUY2 (FRAME_GET_LINE (dest, 0),
       FRAME_GET_STRIDE (dest), FRAME_GET_Y_LINE (src, 0),
       FRAME_GET_Y_STRIDE (src), FRAME_GET_U_LINE (src, 0),
       FRAME_GET_U_STRIDE (src), FRAME_GET_V_LINE (src, 0),
@@ -1019,7 +981,7 @@ convert_Y444_UYVY (VideoConvert * convert, GstVideoFrame * dest,
   gint width = convert->width;
   gint height = convert->height;
 
-  cogorc_convert_Y444_UYVY (FRAME_GET_LINE (dest, 0),
+  video_convert_orc_convert_Y444_UYVY (FRAME_GET_LINE (dest, 0),
       FRAME_GET_STRIDE (dest), FRAME_GET_Y_LINE (src, 0),
       FRAME_GET_Y_STRIDE (src), FRAME_GET_U_LINE (src, 0),
       FRAME_GET_U_STRIDE (src), FRAME_GET_V_LINE (src, 0),
@@ -1033,7 +995,7 @@ convert_Y444_AYUV (VideoConvert * convert, GstVideoFrame * dest,
   gint width = convert->width;
   gint height = convert->height;
 
-  cogorc_convert_Y444_AYUV (FRAME_GET_LINE (dest, 0),
+  video_convert_orc_convert_Y444_AYUV (FRAME_GET_LINE (dest, 0),
       FRAME_GET_STRIDE (dest), FRAME_GET_Y_LINE (src, 0),
       FRAME_GET_Y_STRIDE (src), FRAME_GET_U_LINE (src, 0),
       FRAME_GET_U_STRIDE (src), FRAME_GET_V_LINE (src, 0),
@@ -1048,7 +1010,7 @@ convert_AYUV_ARGB (VideoConvert * convert, GstVideoFrame * dest,
   gint width = convert->width;
   gint height = convert->height;
 
-  cogorc_convert_AYUV_ARGB (FRAME_GET_LINE (dest, 0),
+  video_convert_orc_convert_AYUV_ARGB (FRAME_GET_LINE (dest, 0),
       FRAME_GET_STRIDE (dest), FRAME_GET_LINE (src, 0),
       FRAME_GET_STRIDE (src), width, height);
 }
@@ -1060,7 +1022,7 @@ convert_AYUV_BGRA (VideoConvert * convert, GstVideoFrame * dest,
   gint width = convert->width;
   gint height = convert->height;
 
-  cogorc_convert_AYUV_BGRA (FRAME_GET_LINE (dest, 0),
+  video_convert_orc_convert_AYUV_BGRA (FRAME_GET_LINE (dest, 0),
       FRAME_GET_STRIDE (dest), FRAME_GET_LINE (src, 0),
       FRAME_GET_STRIDE (src), width, height);
 }
@@ -1072,7 +1034,7 @@ convert_AYUV_ABGR (VideoConvert * convert, GstVideoFrame * dest,
   gint width = convert->width;
   gint height = convert->height;
 
-  cogorc_convert_AYUV_ABGR (FRAME_GET_LINE (dest, 0),
+  video_convert_orc_convert_AYUV_ABGR (FRAME_GET_LINE (dest, 0),
       FRAME_GET_STRIDE (dest), FRAME_GET_LINE (src, 0),
       FRAME_GET_STRIDE (src), width, height);
 }
@@ -1084,7 +1046,7 @@ convert_AYUV_RGBA (VideoConvert * convert, GstVideoFrame * dest,
   gint width = convert->width;
   gint height = convert->height;
 
-  cogorc_convert_AYUV_RGBA (FRAME_GET_LINE (dest, 0),
+  video_convert_orc_convert_AYUV_RGBA (FRAME_GET_LINE (dest, 0),
       FRAME_GET_STRIDE (dest), FRAME_GET_LINE (src, 0),
       FRAME_GET_STRIDE (src), width, height);
 }
@@ -1101,14 +1063,14 @@ convert_I420_BGRA (VideoConvert * convert, GstVideoFrame * dest,
   if (quality > 3) {
     for (i = 0; i < height; i++) {
       if (i & 1) {
-        cogorc_convert_I420_BGRA_avg (FRAME_GET_LINE (dest, i),
+        video_convert_orc_convert_I420_BGRA_avg (FRAME_GET_LINE (dest, i),
             FRAME_GET_Y_LINE (src, i),
             FRAME_GET_U_LINE (src, i >> 1),
             FRAME_GET_U_LINE (src, (i >> 1) + 1),
             FRAME_GET_V_LINE (src, i >> 1),
             FRAME_GET_V_LINE (src, (i >> 1) + 1), width);
       } else {
-        cogorc_convert_I420_BGRA (FRAME_GET_LINE (dest, i),
+        video_convert_orc_convert_I420_BGRA (FRAME_GET_LINE (dest, i),
             FRAME_GET_Y_LINE (src, i),
             FRAME_GET_U_LINE (src, i >> 1),
             FRAME_GET_V_LINE (src, i >> 1), width);
@@ -1116,7 +1078,7 @@ convert_I420_BGRA (VideoConvert * convert, GstVideoFrame * dest,
     }
   } else {
     for (i = 0; i < height; i++) {
-      cogorc_convert_I420_BGRA (FRAME_GET_LINE (dest, i),
+      video_convert_orc_convert_I420_BGRA (FRAME_GET_LINE (dest, i),
           FRAME_GET_Y_LINE (src, i),
           FRAME_GET_U_LINE (src, i >> 1),
           FRAME_GET_V_LINE (src, i >> 1), width);
