@@ -124,15 +124,21 @@ struct _GstVideoTestSrc {
 
   /* video state */
   GstVideoInfo info;
-
-  char *format_name;
-  struct format_list_struct *format;
+  gboolean bayer;
+  gint x_invert;
+  gint y_invert;
 
   /* private */
   gint64 timestamp_offset;              /* base offset */
+
+  /* running time and frames for current caps */
   GstClockTime running_time;            /* total running time */
   gint64 n_frames;                      /* total frames sent */
 
+  /* previous caps running time and frames */
+  GstClockTime accum_rtime;              /* accumulated running_time */
+  gint64 accum_frames;                  /* accumulated frames */
+
   /* zoneplate */
   gint k0;
   gint kx;
@@ -161,6 +167,7 @@ struct _GstVideoTestSrc {
   guint8 *tmpline_u8;
   guint8 *tmpline;
   guint8 *tmpline2;
+  guint16 *tmpline_u16;
 };
 
 struct _GstVideoTestSrcClass {
