@@ -35,6 +35,48 @@
 static GType gst_red_video_src_get_type (void);
 static GType gst_codec_src_get_type (void);
 
+GST_START_TEST (test_uri)
+{
+  GstElement *playbin, *fakesink;
+  gchar *uri;
+
+  fail_unless (gst_element_register (NULL, "redvideosrc", GST_RANK_PRIMARY,
+          gst_red_video_src_get_type ()));
+
+  playbin = gst_element_factory_make ("playbin", "playbin");
+  fail_unless (playbin != NULL, "Failed to create playbin element");
+
+  fakesink = gst_element_factory_make ("fakesink", "fakesink");
+  fail_unless (fakesink != NULL, "Failed to create fakesink element");
+
+  g_object_set (playbin, "video-sink", fakesink, NULL);
+
+  g_object_set (playbin, "uri", "redvideo://", NULL);
+  g_object_get (playbin, "uri", &uri, NULL);
+
+  fail_unless_equals_string (uri, "redvideo://");
+  g_free (uri);
+
+  g_object_get (playbin, "current-uri", &uri, NULL);
+  fail_unless_equals_string (uri, NULL);
+
+  fail_unless_equals_int (gst_element_set_state (playbin, GST_STATE_PAUSED),
+      GST_STATE_CHANGE_ASYNC);
+  fail_unless_equals_int (gst_element_get_state (playbin, NULL, NULL, -1),
+      GST_STATE_CHANGE_SUCCESS);
+
+  g_object_get (playbin, "uri", &uri, NULL);
+  fail_unless_equals_string (uri, NULL);
+  g_object_get (playbin, "current-uri", &uri, NULL);
+  fail_unless_equals_string (uri, "redvideo://");
+  g_free (uri);
+
+  gst_element_set_state (playbin, GST_STATE_NULL);
+  gst_object_unref (playbin);
+}
+
+GST_END_TEST;
+
 /* make sure the audio sink is not touched for video-only streams */
 GST_START_TEST (test_sink_usage_video_only_stream)
 {
@@ -610,7 +652,7 @@ gst_red_video_src_class_init (GstRedVideoSrcClass * klass)
 
   gst_element_class_add_pad_template (element_class,
       gst_static_pad_template_get (&src_templ));
-  gst_element_class_set_details_simple (element_class,
+  gst_element_class_set_metadata (element_class,
       "Red Video Src", "Source/Video", "yep", "me");
 
   pushsrc_class->create = gst_red_video_src_create;
@@ -712,7 +754,7 @@ gst_codec_src_class_init (GstCodecSrcClass * klass)
 
   gst_element_class_add_pad_template (element_class,
       gst_static_pad_template_get (&src_templ));
-  gst_element_class_set_details_simple (element_class,
+  gst_element_class_set_metadata (element_class,
       "Codec Src", "Source/Video", "yep", "me");
 
   pushsrc_class->create = gst_codec_src_create;
@@ -736,6 +778,7 @@ playbin_suite (void)
   suite_add_tcase (s, tc_chain);
 
 #ifndef GST_DISABLE_REGISTRY
+  tcase_add_test (tc_chain, test_uri);
   tcase_add_test (tc_chain, test_sink_usage_video_only_stream);
   tcase_add_test (tc_chain, test_suburi_error_wrongproto);
   tcase_add_test (tc_chain, test_suburi_error_invalidfile);
